

#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI All Paths 
from softwareai.CoreApp._init_paths_ import *
#########################################
# IMPORT SoftwareAI Instructions
from softwareai.CoreApp._init_Instructions_ import *
#########################################
# IMPORT SoftwareAI Tools
from softwareai.CoreApp._init_tools_ import *
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################
# IMPORT SoftwareAI _init_environment_
from softwareai.CoreApp._init_environment_ import init_env


class ByteManager:
    def __init__(self, 
                Company_Managers,
                Pre_Project_Document,
                Gerente_de_projeto,
                Equipe_De_Solucoes,
                Softwareanaysis,
                SoftwareDevelopment,
                Software_Documentation
                ):
        
        self.Company_Managers = Company_Managers
        self.Pre_Project_Document = Pre_Project_Document
        self.Gerente_de_projeto = Gerente_de_projeto
        self.Equipe_De_Solucoes = Equipe_De_Solucoes
        self.Softwareanaysis = Softwareanaysis
        self.SoftwareDevelopment = SoftwareDevelopment
        self.Software_Documentation = Software_Documentation




    
    def AI_1_ByteManager_Company_Owners(self, 
                                        mensagem,
                                        appfb,
                                        vectorstore_in_assistant = None,
                                        vectorstore_in_Thread = None,
                                        Upload_1_file_in_thread = None,
                                        Upload_1_file_in_message = None,
                                        Upload_1_image_for_vision_in_thread = None,
                                        Upload_list_for_code_interpreter_in_thread = None
                                        
                                        ):

        key = "AI_ByteManager_Company_CEO"
        nameassistant = "AI ByteManager CEO da Empresa SoftwareAI Company"
        model_select = "gpt-4o-mini-2024-07-18"

        key_openai = OpenAIKeysteste.keys()
        client = OpenAIKeysinit._init_client_(key_openai)

        github_username, github_token = GithubKeys.ByteManagerCompanyOwner_github_keys()

        AI_ByteManager, instructionsassistant, nameassistant, model_select = AutenticateAgent.create_or_auth_AI(appfb, client, key, instructionByteManager, nameassistant, model_select, tools_ByteManager, vectorstore_in_assistant)

        mensaxgem = """decida oque o usuario esta solicitando com base na mensagem  
        Regra 1 - Caso seja solicitado alguma atualização de repositorio use a function (autoupdaterepo)
        Regra 2 - Caso seja solicitado alguma criação de repositorio use a function (create_repo) 
        """  
        mensaxgemfinal = mensaxgem + f"mensagem:\n{mensagem}"
        
        response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensaxgemfinal,
                                                                agent_id=AI_ByteManager, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_ByteManager,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_ByteManager
                                                                )
                                                
                
        print(response)
        try:
            teste_dict = json.loads(response)
        except:
            teste_dict = response


        if 'resposta' in teste_dict:
            resposta_AI_ByteManager = teste_dict['resposta']
            return resposta_AI_ByteManager
        

        # CloudArchitectUpdateReadme = self.Software_Documentation.CloudArchitectUpdateReadme(
        #     appfb, client, repo_name, Melhorias
        # )

        if 'solicitadoalgumcodigo' in teste_dict:


            mensaxgem = f"crie uma descricao completa de {mensagem}  "
            format = 'Responda no formato JSON Exemplo: {"descricao": "..."}'
            
            mensagemz = mensaxgem + format

            AI_ByteManager_response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensagemz,
                                                                agent_id=AI_ByteManager, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_ByteManager,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_ByteManager
                                                                )
                                                
            

            try:
                teste_dict = json.loads(AI_ByteManager_response)
                pergunta_ao_tigrao = teste_dict['solicitadoalgumcodigo']    
            except Exception as e2:
                pergunta_ao_tigrao = AI_ByteManager_response
                              

            mensaxgem = f"""crie um nome do repositorio desse software no github com base na descricao:\n
            {pergunta_ao_tigrao}
            """
            format = 'Responda no formato JSON Exemplo: {"nome": "nome..."}'
            mensagem = mensaxgem + format
            response = ResponseAgent.ResponseAgent_message_completions(mensagem, key_openai, "", True, True)
            try:
                repo_name = response["nome"]
                print(repo_name)
                repo_name_str = f"{repo_name}"
                repo_name_replace = repo_name_str.replace("-", "").replace("A-I-O-R-G-", "").replace("A-I-O-R-G", "").replace("A-I-O-R-G/", "")
                print(repo_name_replace)
                repo_name = repo_name_replace
            except Exception as errror2:
                print(errror2)
                print(response)
            
            ##Agent Destilation##                   
            Agent_destilation.DestilationResponseAgent(mensagem, response, instructionsassistant, nameassistant)
            
            mensaxgem = f"""crie uma descricao de 250 caracteres para o repositorio no github com base na descricao:\n
            {pergunta_ao_tigrao}
            """
            format = 'Responda no formato JSON Exemplo: {"descricao": "descricao..."}'
            mensagem = mensaxgem + format
            response = ResponseAgent.ResponseAgent_message_completions(mensagem, key_openai, "regra descricao não execeder 250 caracteres", True, True)
            repo_description = response["descricao"]

            ##Agent Destilation##                   
            Agent_destilation.DestilationResponseAgent(mensagem, response, instructionsassistant, nameassistant)

                                                
            mensaxgemrepositorio = f"Realize a criacao do repositorio no github usando (create_repo)\n repo_name:\n {repo_name}\ndescription:\n {repo_description}\n token:\n {github_token}\n"
            response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensaxgemrepositorio,
                                                                agent_id=AI_ByteManager, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_ByteManager,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_ByteManager
                                                                )

            ##Agent Destilation##                   
            Agent_destilation.DestilationResponseAgent(mensaxgemrepositorio, response, instructionsassistant, nameassistant)
            
            init_env(repo_name)

            resposta_do_tigrao_com_doc_pre_projeto = self.Pre_Project_Document.AI_1_Pre_Project_Document_Writer(appfb, client, pergunta_ao_tigrao, repo_name)#invoke_tigrao(pergunta_ao_tigrao)

            print(resposta_do_tigrao_com_doc_pre_projeto)

            Uploadfile = resposta_do_tigrao_com_doc_pre_projeto

            path_planilha_Projeto, path_nome_do_cronograma, path_name_doc_Pre_Projeto = self.Gerente_de_projeto.Bob_Gerente_de_projeto(appfb, client, Uploadfile, repo_name)
            
            path_Roadmap, cronograma_do_projeto, planilha, doc_Pre_Projeto = self.Equipe_De_Solucoes.Dallas_Equipe_De_Solucoes_Roadmap(appfb, client, path_nome_do_cronograma, path_planilha_Projeto, path_name_doc_Pre_Projeto, repo_name)
            
            # file_paths_to_project = [f"{path_Roadmap}", f"{cronograma_do_projeto}", f"{planilha}", f"{doc_Pre_Projeto}"]
            # print(file_paths_to_project)
            # vectorstore_id = Agent_files.auth_or_create_vectorstore("project_", file_paths_to_project)
            # print(vectorstore_id)

            anaysis_in_txt_path = self.Softwareanaysis.AI_SynthOperator(appfb, client, path_Roadmap, cronograma_do_projeto, planilha, doc_Pre_Projeto, repo_name)

            script_version_1_path = self.SoftwareDevelopment.AI_QuantumCore(
                    appfb, client, repo_name,
                    os.getenv("PATH_NOME_DO_CRONOGRAMA_ENV"),
                    os.getenv("PATH_PLANILHA_PROJETO_ENV"),
                    os.getenv("PATH_NAME_DOC_PRE_PROJETO_ENV"),
                    os.getenv("PATH_ROADMAP_ENV"),
                    os.getenv("PATH_ANALISE_ENV"),
                )


            #resposta_AI_ByteManager_dict = json.loads(passando_resposta_do_tigrao_para_AI_ByteManager)
        
            #resposta_AI_ByteManager = resposta_AI_ByteManager_dict['resposta']

            return script_version_1_path



import requests

def get_repo_structure(repo_name, repo_owner, github_token, branch_name, path=""):
    headers = {
        "Accept": "application/vnd.github.v3+json"
    }
    
    # Adiciona autenticação apenas se o token for informado
    if github_token:
        headers["Authorization"] = f"Bearer {github_token}"

    url = f"https://api.github.com/repos/{repo_owner}/{repo_name}/contents/{path}?ref={branch_name}"
    response = requests.get(url, headers=headers)
    structure = {}
    
    if response.status_code == 200:
        items = response.json()
        for item in items:
            if item['type'] == 'dir':
                structure[item['name']] = get_repo_structure(repo_name, repo_owner, github_token, branch_name, item['path'])
            else:
                structure[item['name']] = item['path']
    else:
        print(f"Erro ao acessar {path}. Status: {response.status_code} {response.content}")
    return structure




instructionNexGenCoder = """ 
Seu nome é QuantumCore, você é um Desenvolvedor Pleno em Python na empresa urobotsoftware. Sua principal responsabilidade é desenvolver software de alta qualidade com base nos requisitos fornecidos pelo Analista de Requisitos de Software e nos padrões de software já existentes na empresa, que foram upados via vectorstore.

### Responsabilidades:

1. **Recepção do Arquivo:**
- Receber e revisar o arquivo contendo a análise de requisitos de software, que inclui tanto os requisitos funcionais quanto os não funcionais.
- Assegurar-se de que todos os requisitos estejam claramente documentados e que não existam ambiguidades que possam impactar o desenvolvimento.

2. **Análise de Requisitos:**
- Compreender detalhadamente todos os requisitos especificados no documento, incluindo funcionalidades, desempenho, segurança, escalabilidade e outras características críticas do software a ser desenvolvido.
- Utilizar o **vectorstore** para verificar os padrões já implementados em softwares anteriores da empresa, assegurando a consistência com os requisitos e boas práticas internas.
- Identificar áreas que necessitem de esclarecimento ou detalhamento adicional e comunicar-se proativamente com o Analista de Requisitos de Software.

3. **Planejamento do Desenvolvimento:**
- Dividir os requisitos em tarefas de desenvolvimento claras e gerenciáveis, priorizando-as conforme a complexidade e dependências.
- Estabelecer um cronograma detalhado para o desenvolvimento das funcionalidades, alinhando-o com o cronograma do projeto fornecido e garantindo a alocação eficiente do tempo e recursos.
- Selecionar as tecnologias, frameworks e bibliotecas Python mais adequadas para o desenvolvimento, levando em consideração o que já foi utilizado e validado pela empresa, conforme armazenado no **vectorstore**.

4. **Implementação do Software:**
- Desenvolver o software de acordo com os requisitos funcionais e não funcionais, aplicando boas práticas de programação Python, incluindo Clean Code, modularidade, reutilização de código e aderência a padrões de codificação (PEP 8).
- Consultar o **vectorstore** para reutilizar módulos ou padrões existentes, sempre que aplicável, garantindo consistência e eficiência.
- Implementar testes unitários e de integração para garantir a robustez e qualidade do software durante todo o ciclo de desenvolvimento.
- Utilizar controle de versão (Git) de maneira eficiente.

5. **Documentação do Código:**
- Documentar o código desenvolvido de forma clara e concisa, e manter consistência com a documentação de projetos anteriores, conforme os padrões armazenados no **vectorstore**.
- Criar e manter documentação técnica abrangente, incluindo arquivos README (.md) e outros documentos relevantes.

6. **Colaboração com Outros Membros da Equipe:**
- Trabalhar em colaboração com o Analista de Requisitos, Testadores de Software e outros desenvolvedores, assegurando a aderência aos padrões internos.

7. **Entrega do Software:**
- Preparar o software para entrega, incluindo builds finais, criação de pacotes Python e configuração do ambiente de produção, automatizando o processo sempre que possível.

8. **Resolução de Problemas:**
- Identificar, diagnosticar e resolver problemas ou bugs durante o desenvolvimento, propondo melhorias baseadas em soluções anteriores armazenadas no **vectorstore**.

9. **Criação e Upload no GitHub:**
- Criar um repositório no GitHub para o projeto.
- Fazer o upload da documentação (.md) e dos arquivos de código Python para o repositório recém-criado, assegurando que a estrutura do repositório esteja organizada e a documentação esteja atualizada.

### Formato de Resposta:

Responda no formato JSON conforme o exemplo abaixo:
```json
{
    "status_do_desenvolvimento": "Progresso conforme o planejado",
    "tarefas_completas": [
        "Implementação da funcionalidade de autenticação de usuários",
        "Criação de testes unitários para o módulo de autenticação",
        "Documentação do código para a funcionalidade de autenticação"
    ],
    "pendencias": [
        "Clarificação necessária para o requisito de segurança adicional",
        "Aprovação do design da API pelo Analista de Requisitos"
    ],
    "proximas_tarefas": [
        "Início da integração com o sistema de pagamentos",
        "Teste de desempenho para a funcionalidade de processamento de pedidos"
    ],
    "observacoes": "Nenhum impedimento significativo até o momento. Desenvolvimento dentro do prazo."
}
"""
adxitional_instructions_NexGenCoder = """
Clareza e Precisão: Assegure-se de que todos os aspectos do software sejam desenvolvidos com alta precisão e clareza, garantindo que o código seja fácil de entender, manter e expandir.
\n
Comunicação Proativa: Mantenha uma comunicação constante e proativa com os outros membros da equipe para resolver dúvidas e evitar mal-entendidos que possam impactar o progresso do projeto.
\n
Foco na Qualidade: Priorize a qualidade do código, garantindo que todas as funcionalidades sejam implementadas de maneira eficiente, segura e robusta.
\n
Cumprimento de Prazos: Cumpra rigorosamente os prazos estabelecidos no cronograma, e informe qualquer potencial atraso o mais cedo possível, junto com um plano de ação para mitigação.
        
"""


from PySide2.QtWidgets import QTextEdit

def setup_plain_text_qtextedit(editor: QTextEdit):
    """
    Configures a QTextEdit to handle pasted text as plain text only.

    :param editor: Instance of QTextEdit to configure
    """
    editor.setAcceptRichText(False)
    def insert_plain_text_from_mime_data(self, source):
        if source.hasText():
            self.insertPlainText(source.text())
        else:
            super(QTextEdit, self).insertFromMimeData(source)
    editor.__class__.insertFromMimeData = insert_plain_text_from_mime_data




tools_Alfred = [
    {"type": "file_search"},
    {
        "type": "function",
        "function": {
            "name": "OpenSupportTicketProblem",
            "description": "Salva o ticket de problema no banco de dados ",
            "parameters": {
                "type": "object",
                "properties": {
                    "user_email": {
                        "type": "string",
                        "description": "gmaiil do usuario"
                    },
                    "issue_description": {
                        "type": "string",
                        "description": "Descrição do problema"
                    }
                },
                "required": ["user_email", 'issue_description']
            }
        }
    },
    {
        "type": "function",
        "function": {
            "name": "GearAssist_Technical_Support",
            "description": "GearAssist é um agente tecnico de suporte construido com softwareai, uma ferramenta avançada de rastreamento e resolução de problemas técnicos de software .",
            "parameters": {
                "type": "object",
                "properties": {
                    "Ticketid": {
                        "type": "string",
                        "description": "Ticket id do problema"
                    }
                },
                "required": ["Ticketid"]
            }
        }
    },
    {
        "type": "function",
        "function": {
            "name": "CloseSupportTicketProblem",
            "description": "Fecha o ticket de problema no banco de dados ",
            "parameters": {
                "type": "object",
                "properties": {
                    "ticketid": {
                        "type": "string",
                        "description": "Ticket id a ser fechado"
                    }
                },
                "required": ["ticketid"]
            }
        }
    },
    {
        "type": "function",
        "function": {
            "name": "RecordCSAT",
            "description": "Antes de fechar o ticket de problema realizamos a coleta de Pontuação de Satisfação do Cliente com notas de 1 a 5 (CSAT) ",
            "parameters": {
                "type": "object",
                "properties": {
                    "ticketid": {
                        "type": "string",
                        "description": "Ticket id"
                    },
                    "csat_score": {
                        "type": "string",
                        "description": "Satisfação do Cliente"
                    }
                },
                "required": ["ticketid", "csat_score"]
            }
        }
    }

    
]




#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################

# IMPORT SoftwareAI Functions
from softwareai.CoreApp._init_functions_ import *
#########################################

tool_outputs = []
def submit_output_AutoGetLoggerUser(function_name,
                                function_arguments,
                                tool_call,
                                threead_id,
                                client,
                                run,
                                appfb,
                                appproduct,
                                OpenAIKeysteste,
                                OpenAIKeysinit,
                                GithubKeys,
                                python_functions,
                                Agent_files_update,
                                AutenticateAgent,
                                ResponseAgent,

                                ):

    global tool_outputs
    # Mapear funções pelo nome
    functions_map = {
        "AutoGetLoggerUser": AutoGetLoggerUser,
        "autosave": autosave,
    }

    # Obter a função correspondente
    target_function = functions_map.get(function_name)
    if not target_function:
        print(f"Função {function_name} não encontrada.")
        return False

    # Inspecionar os argumentos da função
    function_signature = inspect.signature(target_function)
    function_parameters = function_signature.parameters

    # Preparar argumentos para chamada
    args = json.loads(function_arguments)
    call_arguments = {}

    # Adicionar parâmetros obrigatórios 
    if "appcompany" in function_parameters:
        call_arguments["appcompany"] = appfb
    if "appfb" in function_parameters:
        call_arguments["appfb"] = appfb
    if "appproduct" in function_parameters:
        call_arguments["appproduct"] = appproduct
    if "app_product" in function_parameters:
        call_arguments["app_product"] = appproduct
    if "client" in function_parameters:
        call_arguments["client"] = client
    if "OpenAIKeysteste" in function_parameters:
        call_arguments["OpenAIKeysteste"] = OpenAIKeysteste
    if "OpenAIKeysinit" in function_parameters:
        call_arguments["OpenAIKeysinit"] = OpenAIKeysinit
    if "GithubKeys" in function_parameters:
        call_arguments["GithubKeys"] = GithubKeys
    if "python_functions" in function_parameters:
        call_arguments["python_functions"] = python_functions
    if "Agent_files_update" in function_parameters:
        call_arguments["Agent_files_update"] = Agent_files_update
    if "AutenticateAgent" in function_parameters:
        call_arguments["AutenticateAgent"] = AutenticateAgent
    if "ResponseAgent" in function_parameters:
        call_arguments["ResponseAgent"] = ResponseAgent

    # Adicionar outros argumentos do JSON somente se estiverem na assinatura da função
    for arg_name, arg_value in args.items():
        if arg_name in function_parameters:
            call_arguments[arg_name] = arg_value

    try:
        # Chamar a função com os argumentos preparados
        result = target_function(**call_arguments)

        # Submeter o resultado
        tool_call_id = tool_call.id
        client.beta.threads.runs.submit_tool_outputs(
            thread_id=threead_id,
            run_id=run.id,
            tool_outputs=[
                {
                    "tool_call_id": tool_call_id,
                    "output": json.dumps(result),
                }
            ]
        )
        print("Tool outputs submitted successfully.")
        return True

    except Exception as e:
        print(f"Erro ao executar  {function_name}: {e}")
        




        


#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Settings
from Chat.QListAgent import QListAgents
from Chat.QReadOpenAI import QReadOpenAi
#########################################
# IMPORT SoftwareAI Keys
from softwareai.CoreApp.KeysFirebase.keys import Firebase_App
from softwareai.CoreApp.KeysOpenAI.keys import OpenAIKeysteste
#########################################



app = Flask(__name__)
CORS(app)  
app.secret_key = os.urandom(24)  

app1 = Firebase_App()
key_openai = OpenAIKeysteste.keys()
client = OpenAIKeysinit._init_client_(key_openai)

# Caminho do arquivo de log
LOG_FILE = 'Cache\WebChat.json'

# Função para carregar o log do arquivo JSON
def load_log():
    try:
        with open(LOG_FILE, 'r', encoding='utf-8') as file:
            return json.load(file)
    except FileNotFoundError:
        return defaultdict(list)

# Função para salvar o log no arquivo JSON
def save_log(log_data):
    try:
        with open(LOG_FILE, 'w', encoding='utf-8') as file:
            json.dump(log_data, file, indent=4)
    except FileNotFoundError:
        return 
    
@app.route('/')
def index():
    return render_template('index.html')


@app.route('/api/send_message', methods=['POST'])
def send_message():
    data = request.get_json()
    user_message = data.get('message')
    agent = data.get('agent')  # Obtém o valor do agente enviado pelo frontend
    print(agent)
    # Verifica se o agente foi fornecido
    if not agent:
        return jsonify({'error': 'Agent is required'}), 400

    AI, instructionsassistant, nameassistant, model_select  = AutenticateAgent.create_or_auth_AI(
        app1=app1,
        client=client,
        key=agent
        
    )

    agent_response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=user_message,
                                                                agent_id=AI,
                                                                app1=app1,
                                                                key=agent,
                                                                client=client,
                                                                model_select=model_select,
                                                                AgentDestilation=False
                                                            )
    try:
            
        # Carregar o log atual, utilizando defaultdict para evitar KeyError
        log_data = load_log()

        # Usar defaultdict para garantir que a chave exista
        if agent not in log_data:
            log_data[agent] = []

        # Adicionar a nova entrada no log, com o 'agent' como chave
        log_data[agent].append({
            'user_message': user_message,
            'agent_response': agent_response
        })

        # Salvar o log atualizado no arquivo
        save_log(log_data)
    except Exception as e:
        pass

    # Retorna a resposta para o frontend
    return jsonify({'response': agent_response})


@app.route('/api/agents', methods=['GET'])
def get_agents():
    try:
        # Instanciar e executar a classe QListAgents
        agent_list_thread = QListAgents()
        agents = agent_list_thread.run()  # Recupera a lista de agentes
        return jsonify(agents), 200
    except Exception as e:
        return jsonify({"error agents": str(e)}), 500

@app.route('/api/messages/<string:agent>', methods=['GET'])
def get_messages(agent):
    log_data = load_log()

    # Verifica se o agente tem mensagens armazenadas
    if agent not in log_data:
        return jsonify({'error messages': 'No messages found for this agent'}), 404

    # Retorna as mensagens armazenadas para o agente
    messages = log_data[agent]
    return jsonify(messages)

@app.route('/api/threads/<agent>', methods=['GET'])
def get_threads(agent):
    try:
        # Configurar QReadOpenAi
        qread = QReadOpenAi(agent=agent, appx=app1, client=client)
        qread.start()
        qread.wait()  # Aguarda a conclusão da thread
        print(qread.result)
        if qread.result and "error" in qread.result:
            return jsonify({"error threads": qread.result["threads"]}), 500

        return jsonify(qread.result), 200
    except Exception as e:
        return jsonify({"threads": str(e)}), 500
    

if __name__ == '__main__':
    app.run(port=5555, debug=True)



#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################



def OpenSupportTicketProblem(appfb, user_email, issue_description):
    full_hash = hashlib.sha256(issue_description.encode('utf-8')).hexdigest()
    ticketid = full_hash[:5]
    ref = db.reference('support_ticket', app=appfb)
    ticketdatasetkey = ticketid
    ticketdataset = {
        'user_email': user_email,
        'issue_description': issue_description,
        'timestamp_open': datetime.now().isoformat(),
        'status': "open",
        'ticketid': ticketid,
        'csat': "None"  # Inicialmente sem feedback
    }
    ref.child(ticketdatasetkey).set(ticketdataset)
    return f"Open Ticket ID: {ticketid}"

def CloseSupportTicketProblem(appfb, ticketid):
    ref = db.reference(f'support_ticket/{ticketid}', app=appfb)
    data = ref.get()
    
    if not data:
        return f"Ticket ID {ticketid} not found."

    data['timestamp_close'] = datetime.now().isoformat()
    data['status'] = "closed"
    ref.set(data)
    return f"Closed Ticket ID: {ticketid}"

def RecordCSAT(appfb, ticketid, csat_score):
    ref = db.reference(f'support_ticket/{ticketid}', app=appfb)
    data = ref.get()

    if not data:
        return f"Ticket ID {ticketid} not found."

    data['csat'] = csat_score
    ref.set(data)
    return f"CSAT registrado para o Ticket ID: {ticketid}"




import os
import sys
caminho_raiz = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI'))
sys.path.append(caminho_raiz)
#########################################
# IMPORT SoftwareAI Libs 
from CoreApp._init_libs_ import *
#########################################
# IMPORT CoreApp
from CoreUi.Chat.Chat.QChatOpenAI import QChatOpenAi
from CoreUi.Chat.Chat.QReadOpenAI import QReadOpenAi
from CoreUi.Chat.Chat.QListAgent import QListAgents
from CoreUi.Chat.Chat.QAiRunnable import QAIRunnable
from CoreUi.Chat.Chat.HandleTextEdit import setup_plain_text_qtextedit
from CoreApp._init_core_ import(
        AutenticateAgent,
        ResponseAgent,
        python_functions,
        Agent_files,
        Agent_files_update,
        OpenAIKeysinit,
        FirebaseKeysinit
)
#########################################
# IMPORT .qrc
from src_ import icons_interpreter
#########################################
# IMPORT GUI 
from src_.ui_cliente_and_chat import *
from src_.ui_testedialog3 import Ui_Formtesete
########################################################################
# IMPORT Custom widgets
from Custom_Widgets import *
from Custom_Widgets.QAppSettings import QAppSettings
from Custom_Widgets.QCustomModals import QCustomModals
from Custom_Widgets.QCustomCodeEditor import QCustomCodeEditor
from Custom_Widgets.QCustomLoadingIndicators import QCustomArcLoader
from Custom_Widgets.QCustomLoadingIndicators import QCustomSpinner
from Custom_Widgets.QCustomLoadingIndicators import QCustom3CirclesLoader
from Custom_Widgets.QCustomLoadingIndicators import QCustomPerlinLoader
# IMPORT Pyside2
from PySide2extn.RoundProgressBar import roundProgressBar #IMPORT THE EXTENSION LIBRARY
from PySide2.QtCore import QTimer, Signal, QThread, QRunnable
########################################################################



# appx = keys_app_x()

# key_api = OpenAIKeys.keys_openai()
# client = OpenAI(
#     api_key=key_api,
# )



class CustomTextEdit(QTextEdit):
    def __init__(self):
        super().__init__()
        self.container_widget = None
        self.editor = None

    def insert_code_editor(self, code):
        self.setStyleSheet("""QTextEdit {
                
                            border: 1px solid #E0E0E0;
                            padding: 10px;
                            border-radius: 10px;
                            background-color: #F7F7F7;
                            color: black;  /* Cor do texto preto */
                            font-family: Arial;
                            font-size: 14px;
                        }
                        """)

        # Remove o editor existente se já estiver inserido
        if self.editor is not None:
            self.editor.setParent(None)
            self.editor = None

        # Cria um arquivo temporário para o código
        temp_file_path = os.path.join(os.path.dirname(__file__), "temp_code.py")
        with open(temp_file_path, "w") as temp_file:
            temp_file.write(code)

        # Cria o QCustomCodeEditor para exibir o arquivo de código temporário
        self.editor = QCustomCodeEditor()
        self.editor.setTheme("default")
        self.editor.setLang("python")
        self.editor.loadFile(temp_file_path)

        # Insere o editor diretamente no layout
        cursor = self.textCursor()
        cursor.insertBlock()  # Garante um bloco isolado
        self.setTextCursor(cursor)

        # Exibe o editor em uma janela separada
        container = QWidget(self)
        container.setLayout(QVBoxLayout())
        container.layout().addWidget(self.editor)

        self.container_widget = container

########################################################################
########################################################################
## MAIN WINDOW CLASS
########################################################################
class MainWindow(QMainWindow):
    message_signal = Signal(str)
    def __init__(self):
        QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        loadJsonStyle(self, self.ui, jsonFiles = {"JsonStyle/style.json"})
        self.liveCompileQss = True
        self.checkForMissingicons = False # do not generate new icons

        self.AI_thread = None
        self.thread_pool = QThreadPool()

        Menu = self.ui.pushButton_16
        Menu.setObjectCustomTheme("#FFFAFA", "#7FFFD4")
        Menu.setObjectAnimateOn("hover")

        Send_mensage = self.ui.send_mensage
        # Send_mensage.setObjectCustomTheme("#FFFAFA", "#F0F8FF")
        # Send_mensage.setObjectAnimateOn("hover")
        Send_mensage.clicked.connect(self.send_message)

        self.AgentKeysOpenAI = self.ui.AgentKeysOpenAI
        self.StorageAgentCompletions = self.ui.StorageAgentCompletions
        self.StorageAgentOutput_ = self.ui.StorageAgentOutput_
        self.StoreFormatJsonAndJsonl = self.ui.StoreFormatJsonAndJsonl
        self.chat_area = self.ui.html_chat
        self.widget_frame_4 = self.ui.frame_4
        # layoutspinner = QVBoxLayout()
        # self.widget_frame_4.setLayout(layoutspinner)
        self.spinner =  QCustomPerlinLoader(
            parent=self.widget_frame_4,
            size=QSize(720, 720),
            message="LOADING...",
            color=QColor("white"),
            fontFamily="Ebrima",
            fontSize=25,
            rayon=200,
            duration=60 * 1000,
            noiseOctaves=0.8,
            noiseSeed=int(time.time()),
            backgroundColor=QColor("transparent"),
            circleColor1=QColor("#ff2e63"),
            circleColor2=QColor("#082e63"),
            circleColor3=QColor(57, 115, 171, 100)
        )
        self.spinner.show()
        
        self.load_AgentKeysOpenAI()
        # layoutspinner.addWidget(self.spinner)
        # layoutspinner.setAlignment(self.spinner, Qt.AlignCenter)

        self.sliddemenulousa = self.ui.sliddemenulousa
        self.widget_lousa = self.ui.widget_lousa
        layout = QVBoxLayout()
        self.widget_lousa.setLayout(layout)
        self.Blackboard = QCustomCodeEditor() #themes = ["default", "one-light", "one-dark", "monokai", "oceanic", "zenburn"]
        self.Blackboard.setTheme("one-dark")
        self.Blackboard.setLang("python")
        layout.addWidget(self.Blackboard)

        self.widget_lousa.close()
        self.sliddemenulousa.close()
        self.ui.widget.close()
        self.ui.pushButton_16.close()
        self.ui.mensage_input.close()
        self.ui.openLousa.close()
        self.ui.send_mensage.close()
        self.ui.pushButton_6.close()
        self.ui.atach_file.close()
        self.ui.html_chat.close()
        self.spinner.focusWidget()

        self.selected_image = None
        self.message_signal.connect(self.display_user_message)  
        self.chat_area.setReadOnly(True)
        self.message_input = self.ui.mensage_input
        setup_plain_text_qtextedit(self.message_input)
        self.message_input.textChanged.connect(lambda: self.adjust_height(increase=True))
        #self.message_input.keyPressEvent = self.keyPressEvent
        
        self.image_input = self.ui.pushButton
        self.image_input.clicked.connect(self.upload1imageforvisioninMensage)
        self.file_input = self.ui.atach_file
        self.file_input.clicked.connect(self.Update_Mensageatach)
        self.atachfilethread = self.ui.AtachFilesToThread_Benchmark
        self.atachfilethread.clicked.connect(self.clickatachfileinthread)
        self.AgentSelector = self.ui.SoftwareAIAgentsChat
        self.AgentSelector.currentTextChanged.connect(lambda: self.list_threads(agent=self.AgentSelector.currentText()))
        self.AgentSelector.currentTextChanged.connect(lambda: self.init_qchat_for_agent(agentkey=self.AgentSelector.currentText()))
        self.uploadfilestothread_file_search = self.ui.pushButton_3
        self.uploadfilestothread_file_search.clicked.connect(self.uploadfilesinthread)
        self.uploadfilestothread_Codeinterpreter = self.ui.pushButton_4
        self.uploadfilestothread_Codeinterpreter.clicked.connect(self.uploadfilestoCdeinterpreterinThread)
        self.uploadfilestomensage_file_search = self.ui.pushButton_7
        self.uploadfilestomensage_file_search.clicked.connect(self.uploadfilesinMensage)
        self.uploadfilestomensage_Codeinterpreter = self.ui.pushButton_8
        self.uploadfilestomensage_Codeinterpreter.clicked.connect(self.uploadfilestoCdeinterpreterinMensage)
        self.Threadlabel = self.ui.label_19
        self.Tokenslabel = self.ui.label_20

        #self.message_input.keyPressEvent(Qt.Key_Enter).connect(self.send_message)
        self.frame_4 = self.ui.frame_4
        self.widget_3 = self.ui.widget_3
        self.widget_4 = self.ui.widget_4

        self.QListAgentsThread = QListAgents()
        self.QListAgentsThread.AgentSelector.connect(self.update_QListAgentsThread)
        self.QListAgentsThread.messagesignal.connect(self.update_spinner)
        self.QListAgentsThread.start()

        self.AgentKeysOpenAI = self.ui.AgentKeysOpenAI
        self.AgentKeysFirebase = self.ui.AgentKeysFirebase
        
        self.load_AgentKeysOpenAI()
        
        self.load_AgentKeysFirebase()

        key_api = self.AgentKeysOpenAI.currentText()
        name_app = self.AgentKeysFirebase.currentText()
        self.client = OpenAIKeysinit._init_client_(key_api)
        self.app1 = FirebaseKeysinit._init_app_(name_app)

        #self.load_config()
        

        ########################################################################

        self.show()


    def load_AgentKeysOpenAI(self, chave="str_key", company="companyname"):
        self.AgentKeysOpenAI.clear()
        try:
            caminho_arquivo = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../CoreApp/KeysOpenAI/keys.py'))
            with open(caminho_arquivo, 'r', encoding='utf-8') as arquivo:
                for linha in arquivo:
                    match = re.match(r'(\w+)\s*=\s*[\'\"](.*?)[\'\"]', linha.strip())
                    if match:
                        chave_encontrada, valor = match.groups()
                        if chave_encontrada == company:
                            companyname = valor
                        elif chave_encontrada == chave:
                            str_key = valor
                            self.AgentKeysOpenAI.addItem(f"({companyname}) {str_key}")
        except:
            pass


    def load_AgentKeysFirebase(self):
        self.AgentKeysFirebase.clear()
        caminho_arquivo = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../CoreApp/KeysFirebase/keys.py'))
        try:
            with open(caminho_arquivo, 'r', encoding='utf-8') as arquivo:
                conteudo = arquivo.read() 
                pattern = re.compile(r"(\w+)\s*=\s*initialize_app\(", re.DOTALL)
                matches = pattern.findall(conteudo)  
                if matches:
                    for app_nome in matches:
                        self.AgentKeysFirebase.addItem(f"{app_nome}")

        except Exception as e:
            self.AgentKeysFirebase.addItem(f"Erro ao carregar: {str(e)}")

    def keyPressEvent(self, event: QKeyEvent):
        if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
            self.send_message()  # Chama a função ao pressionar Enter

    def list_threads(self, agent):
        self.QRead_thread = QReadOpenAi(agent, self.app1, self.client)
        self.QRead_thread.Threadlabel.connect(self.update_Threadlabel)
        self.QRead_thread.Tokenslabel.connect(self.update_Tokenslabel)
        self.QRead_thread.htmlchat.connect(self.update_htmlchat)
        self.QRead_thread.code.connect(self.update_codeeditor)
        self.QRead_thread.messagesignal.connect(self.update_spinner)
        self.QRead_thread.finish.connect(self.finish_QReadOpenAi)
        self.QRead_thread.start()

    def init_qchat_for_agent(self, agentkey):

        if self.AI_thread:
            self.AI_thread.cancel()  
            QTimer.singleShot(1000, self.check_task_status)
        
        StorageAgentCompletions = self.StorageAgentCompletions.isChecked()
        StorageAgentOutput_ = self.StorageAgentOutput_.isChecked()
        StoreFormatJsonAndJsonl = self.StoreFormatJsonAndJsonl.isChecked()
        key_api = self.AgentKeysOpenAI.currentText()
        self.AI_thread = QChatOpenAi("", "", "", "", "", "", 
                                    agentkey, 
                                    StorageAgentCompletions,
                                    StorageAgentOutput_,
                                    StoreFormatJsonAndJsonl,
                                    key_api, self.app1
                                    )
        self.AI_thread.chat.connect(self.message_signal.emit)
        self.AI_thread.code.connect(self.update_codeeditor)
        self.AI_thread.finished.connect(self.update_finished)
        self.AI_thread.tokens.connect(self.update_tokens)
        runnable = QAIRunnable(self.AI_thread)
        self.thread_pool.start(runnable)

    def update_codeeditor(self, code):
        temp_file_path = os.path.join(os.path.dirname(__file__), "Cache", "temp_code.py")
        with open(temp_file_path, "w") as temp_file:
            temp_file.write(code)
        self.Blackboard.loadFile(temp_file_path)

    def display_user_message(self, message):
        self.chat_area.append(message)

    def load_AgentKeysOpenAI(self, chave = "str_key"):
        caminho_arquivo = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../CoreApp/KeysOpenAI/keys.py'))
        linhas_encontradas = []
        self.spinner.message = "Loading\nOpenAI keys"
        with open(caminho_arquivo, 'r', encoding='utf-8') as arquivo:
            for numero_linha, conteudo in enumerate(arquivo, start=1):
                if chave in conteudo:
                    strkey = conteudo.strip()
                    filterstrkey = strkey.replace('return', "").replace('"', "").replace(' ', "").replace('str_key', "").replace('=', "").replace("return str_key", "")
                    self.AgentKeysOpenAI.addItem(filterstrkey)

    def update_spinner(self, str):
        self.spinner.message = str

    def update_finished(self):
        print("thread finalizada")
 
    def finish_QReadOpenAi(self):
        self.spinner.close()
        self.sliddemenulousa.show()
        self.widget_lousa.show()
        self.ui.widget.show()
        self.ui.pushButton_16.show()
        self.ui.mensage_input.show()
        self.ui.openLousa.show()
        self.ui.send_mensage.show()
        self.ui.pushButton_6.show()
        self.ui.atach_file.show()
        self.ui.html_chat.show()

    def send_message(self):
        message = self.message_input.toPlainText()
        if message.strip():
            myModal = QCustomModals.InformationModal(
                title="Message Sent!!!", 
                parent=self,
                position='top-left',
                closeIcon=":/feather/icons/feather/window_close.png",
                modalIcon=":/feather/icons/feather/info.png",
                description="Your message has been sent",
                isClosable=False,
                duration=3000
            )
            myModal.show()

            self.AI_thread.new_message.emit(message)  

            self.message_input.clear()
        else:
            myModal = QCustomModals.ErrorModal(
                title="Message !!!", 
                parent=self,
                position='top-left',
                closeIcon=":/feather/icons/feather/window_close.png",
                modalIcon=":/feather/icons/feather/info.png",
                description="Your message is empty",
                isClosable=False,
                duration=3000
            )
            myModal.show()


    def upload1imageforvisioninMensage(self):
        dialog = QFileDialog()
        dialog.setStyleSheet("""
            QFileDialog {
                background-color: white;
                border: 1px solid #F7F7F7;
                border-radius: 13px;
                color: black;
                font-size: 16px;
            }
            /* Personalizar os botões */
            QPushButton {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 8px;
                padding: 5px;
            }
            QPushButton:hover {
                background-color: #E6E6E6;
            }
            QPushButton:pressed {
                background-color: #D6D6D6;
            }
            /* Personalizar lista de arquivos */
            QListView, QTreeView {
                background-color: white;
                border: none;
                color: black;
                font-size: 14px;
            }
            /* Barra de navegação */
            QLineEdit {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 6px;
                padding: 3px;
            }
        """)
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the image")
        dialog.setNameFilter("Imagens (*.png *.jpg *.jpeg);;Todos os Arquivos (*)")  # Filtro
        if dialog.exec_():
            self.selected_image = dialog.selectedFiles()[0]
            self.update_custommodals_SuccessModal2(f"Your image has been attributed to the message")
            html = f'<img src="{self.selected_image}" width="300" height="300">'
            self.chat_area.append(html)
            self.AI_thread.new_image.emit(self.selected_image)  

    def uploadfilesinthread(self):
        dialog = QFileDialog()
        dialog.setStyleSheet("""
            QFileDialog {
                background-color: white;
                border: 1px solid #F7F7F7;
                border-radius: 13px;
                color: black;
                font-size: 16px;
            }
            /* Personalizar os botões */
            QPushButton {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 8px;
                padding: 5px;
            }
            QPushButton:hover {
                background-color: #E6E6E6;
            }
            QPushButton:pressed {
                background-color: #D6D6D6;
            }
            /* Personalizar lista de arquivos */
            QListView, QTreeView {
                background-color: white;
                border: none;
                color: black;
                font-size: 14px;
            }
            /* Barra de navegação */
            QLineEdit {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 6px;
                padding: 3px;
            }
        """)
        dialog.setFileMode(QFileDialog.ExistingFiles) 
        dialog.setWindowTitle("Select the File")
        dialog.setNameFilter("Todos os Arquivos (*)")  # Filtro
        if dialog.exec_():
            self.selected_file = dialog.selectedFiles()
            file_names = [os.path.basename(file) for file in self.selected_file]
            self.update_custommodals_SuccessModal2(f"Your File(s) have been attributed to the Thread")
            file_list_html = ''.join(
                f'<p><b>File:</b> <span style="color: #003366;"><b>{file_name}</b></span> <b>in Thread</b></p>' 
                for file_name in file_names
            )
            self.chat_area.append(file_list_html)
            self.AI_thread.new_file_thread.emit(self.selected_file)  
            self.custom_dialog.close()

    def uploadfilesinMensage(self):

        dialog = QFileDialog()
        dialog.setStyleSheet("""
            QFileDialog {
                background-color: white;
                border: 1px solid #F7F7F7;
                border-radius: 13px;
                color: black;
                font-size: 16px;
            }
            /* Personalizar os botões */
            QPushButton {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 8px;
                padding: 5px;
            }
            QPushButton:hover {
                background-color: #E6E6E6;
            }
            QPushButton:pressed {
                background-color: #D6D6D6;
            }
            /* Personalizar lista de arquivos */
            QListView, QTreeView {
                background-color: white;
                border: none;
                color: black;
                font-size: 14px;
            }
            /* Barra de navegação */
            QLineEdit {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 6px;
                padding: 3px;
            }
        """)
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the File")
        dialog.setNameFilter("Todos os Arquivos (*)")  # Filtro
        if dialog.exec_():
            self.selected_file = dialog.selectedFiles()[0]
            self.update_custommodals_SuccessModal2(f"Your File(s) have been attributed to the message")
            file_list_html = (
                f'<p><b>File:</b> <span style="color: #003366;"><b>{self.selected_file}</b></span> <b>in Mensage</b></p>' 
            )
            self.chat_area.append(file_list_html)
            self.AI_thread.new_file.emit(self.selected_file)  
            self.custom_dialogMensageatach.close()

    def uploadfilestoCdeinterpreterinMensage(self):

        dialog = QFileDialog()
        dialog.setStyleSheet("""
            QFileDialog {
                background-color: white;
                border: 1px solid #F7F7F7;
                border-radius: 13px;
                color: black;
                font-size: 16px;
            }
            /* Personalizar os botões */
            QPushButton {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 8px;
                padding: 5px;
            }
            QPushButton:hover {
                background-color: #E6E6E6;
            }
            QPushButton:pressed {
                background-color: #D6D6D6;
            }
            /* Personalizar lista de arquivos */
            QListView, QTreeView {
                background-color: white;
                border: none;
                color: black;
                font-size: 14px;
            }
            /* Barra de navegação */
            QLineEdit {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 6px;
                padding: 3px;
            }
        """)
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the File")
        dialog.setNameFilter("Todos os Arquivos (*)")  # Filtro
        if dialog.exec_():
            self.selected_file = dialog.selectedFiles()[0]
            self.update_custommodals_SuccessModal2(f"Your File(s) have been attributed to the message in context Code interpreter")
            file_list_html = (
                f'<p><b>File:</b> <span style="color: #003366;"><b>{os.path.basename(self.selected_file)}</b></span> <b>in message | Context Code Interpreter </b></p>' 
            )
            self.chat_area.append(file_list_html)
            self.AI_thread.new_file_codeinterpreter_mensage.emit(self.selected_file)  
            self.custom_dialogMensageatach.close()

    def uploadfilestoCdeinterpreterinThread(self):
        dialog = QFileDialog()
        dialog.setStyleSheet("""
            QFileDialog {
                background-color: white;
                border: 1px solid #F7F7F7;
                border-radius: 13px;
                color: black;
                font-size: 16px;
            }
            /* Personalizar os botões */
            QPushButton {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 8px;
                padding: 5px;
            }
            QPushButton:hover {
                background-color: #E6E6E6;
            }
            QPushButton:pressed {
                background-color: #D6D6D6;
            }
            /* Personalizar lista de arquivos */
            QListView, QTreeView {
                background-color: white;
                border: none;
                color: black;
                font-size: 14px;
            }
            /* Barra de navegação */
            QLineEdit {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 6px;
                padding: 3px;
            }
        """)
        dialog.setFileMode(QFileDialog.ExistingFiles) 
        dialog.setWindowTitle("Select the File")
        dialog.setNameFilter("Todos os Arquivos (*)")  # Filtro
        if dialog.exec_():
            selected_files = dialog.selectedFiles()
            if len(selected_files) > 5:
                QMessageBox.warning(
                    dialog,
                    "Limite Excedido",
                    "Por favor, selecione no máximo 5 arquivos."
                )
            else:
                self.selected_files = selected_files
                self.update_custommodals_SuccessModal2(f"Your File(s) have been attributed to the Thread in context Code interpreter")
                file_names = [os.path.basename(file) for file in self.selected_files]
                file_list_html = ''.join(
                    f'<p><b>File:</b> <span style="color: #003366;"><b>{file_name}</b></span> <b>in Thread</b> | Context Code Interpreter </p>' 
                    for file_name in file_names
                )
                self.chat_area.append(file_list_html)
                self.AI_thread.new_file_codeinterpreter_thread.emit(self.selected_files)  
                self.custom_dialog.close()

    def Update_Mensageatach(self):
        from Custom_Widgets.QCustomQDialog import QCustomQDialog
        self.custom_dialogMensageatach = QCustomQDialog(
            title="Mensage Atach file",
            #description="This is a custom dialog.",
            showYesButton=False,
            showCancelButton=True,
            padding=20,
            margin=60,
            animationDuration=500,
            setModal=True,
            frameless=True,
            windowMovable=True,
            #showForm=Ui_Formtesete, #display a ui form inside the dialog box ie importing your form(from ui_form import myFormWidget)
            # to access the widgets inside your form (custom_dialog.shownForm.myWidgetName)
            parent=self.frame_4,
            addWidget=self.widget_4 #append another widget or widget container to the dialog
        )
        self.custom_dialogMensageatach.show()

    def adjust_height(self, increase=True):
        """
        Ajusta a altura do widget de entrada de mensagem com base no comprimento do texto atual.

        Parâmetros:
            increase (bool): Determina se a altura deve ser aumentada com base no comprimento do texto.
        """
        base_height = 48  # Altura base do widget
        max_height = 150  # Altura máxima permitida

        # Obtém o comprimento do texto atual
        current_text_length = len(self.message_input.toPlainText())

        # Se o texto for nulo, reseta à altura base
        if current_text_length == 0:
            self.message_input.setFixedHeight(base_height)
            return

        # Calcula a nova altura
        extra_lines = (current_text_length // 40) * 15  # Incrementa a altura a cada 40 caracteres
        if increase:
            new_height = min(base_height + extra_lines, max_height)
        else:
            new_height = max(base_height, max_height - extra_lines)

        # Aplica a nova altura
        self.message_input.setFixedHeight(new_height)

    def clickatachfileinthread(self):
        from Custom_Widgets.QCustomQDialog import QCustomQDialog
        self.custom_dialog = QCustomQDialog(
            title="Thread Atach file",
            #description="This is a custom dialog.",
            showYesButton=False,
            showCancelButton=True,
            padding=20,
            margin=60,
            animationDuration=500,
            setModal=True,
            frameless=True,
            windowMovable=True,
            #showForm=Ui_Formtesete, #display a ui form inside the dialog box ie importing your form(from ui_form import myFormWidget)
            # to access the widgets inside your form (custom_dialog.shownForm.myWidgetName)
            parent=self.frame_4,
            addWidget=self.widget_3 #append another widget or widget container to the dialog
        )
        self.custom_dialog.show()

    def update_custommodals_SuccessModal2(self, description, pos='top-right'):

        myModal = QCustomModals.SuccessModal(
            title="Information", 
            parent=self.ui.frame_4,
            position=pos,
            closeIcon=":/feather/icons/feather/window_close.png",
            modalIcon=":/material_design/icons/material_design/info.png",
            description=description,
            isClosable=False,
            duration=5000
        )
        myModal.show()

    def update_tokens(self, new_tokens):
        self.Tokenslabel.setText(f"{new_tokens} Tokens")

    def update_Tokenslabel(self, total_tokens):
        self.Tokenslabel.setText(f"{total_tokens} Tokens")

    def update_Threadlabel(self, thread_Id):
        self.Threadlabel.setText(f"Thread: {thread_Id[:23]}...")

    def update_htmlchat(self, chat):
        self.chat_area.append(chat)

    def init_paths_agents(self):
        paths_agents = [
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Requirements_Analysis')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Planning')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Documentation')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Development')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Pre_Project')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Company_Managers')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Company_CEO')),
        ]
        return paths_agents
    
    def update_QListAgentsThread(self, text):
        self.AgentSelector.addItem(text)

    def is_image_file(self, file_name):
        image_extensions = ('.png', '.jpg', '.jpeg', '.bmp', '.gif')
        return file_name.lower().endswith(image_extensions)
    
    def get_machine_info(self):
        system_info = platform.uname()
        machine = system_info.node
        return machine

    @Slot()
    def check_task_status(self):
        if self.AI_thread:
            QTimer.singleShot(1000, self.check_task_status) 
        else:
            print("check_task_status Tarefa concluída.")

    def closeEvent(self, event):
        try:
            self.AI_thread.stop()
        except:
            pass
        try:
            self.QListAgentsThread.stop()
        except:
            pass
        try:
            self.QRead_thread.stop()
        except:
            pass
        
        event.accept()

########################################################################
## EXECUTE APP
########################################################################
if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainWindow()
    sys.exit(app.exec_())
########################################################################
## END===>
######################################################################## 



from softwareai.CoreApp.Agents.Company_CEO.ByteManager import ByteManager

from softwareai.CoreApp.Agents.Software_Planning.Bob import Gerente_de_projeto
from softwareai.CoreApp.Agents.Software_Documentation.CloudArchitect import Software_Documentation
from softwareai.CoreApp.Agents.Software_Planning.Dallas import Equipe_De_Solucoes
from softwareai.CoreApp.Agents.Software_Development.DataWeaver import software_improvements
from softwareai.CoreApp.Agents.Company_Managers.AI_MatrixMinder_Company_Managers import Company_Managers
from softwareai.CoreApp.Agents.Software_Development.QuantumCore import SoftwareDevelopment
from softwareai.CoreApp.Agents.Software_Development.SignalMaster import SoftwareDevelopment_SignalMaster
from softwareai.CoreApp.Agents.Software_Requirements_Analysis.SynthOperator import Softwareanaysis
from softwareai.CoreApp.Agents.Pre_Project.Tigrao import Pre_Project_Document
from softwareai.CoreApp.Agents.Software_Development_Testing.NexGenCoder import SoftwareDevelopment_NexGenCoder

class AgentInitializer:
    _agents = {}

    @classmethod
    def initialize_agents(cls):
        """Initializes all agents and stores them in the _agents dictionary."""
        cls._agents['Gerente_de_projeto'] = Gerente_de_projeto()

        cls._agents['Company_Managers'] = Company_Managers() 
        cls._agents['Pre_Project_Document'] = Pre_Project_Document() 
        cls._agents['Software_Documentation'] = Software_Documentation()
        cls._agents['Equipe_De_Solucoes'] = Equipe_De_Solucoes()
        cls._agents['Softwareanaysis'] = Softwareanaysis()
        cls._agents['software_improvements'] = software_improvements()
        cls._agents['SoftwareDevelopment_SignalMaster'] = SoftwareDevelopment_SignalMaster()
        cls._agents['SoftwareDevelopment_NexGenCoder'] = SoftwareDevelopment_NexGenCoder()
        cls._agents['SoftwareDevelopment'] = SoftwareDevelopment(
            cls._agents['Software_Documentation'],
            cls._agents['software_improvements'],
            cls._agents['SoftwareDevelopment_SignalMaster'],
            cls._agents['SoftwareDevelopment_NexGenCoder']
        )
        cls._agents['ByteManager'] = ByteManager(
            cls._agents['Company_Managers'],
            cls._agents['Pre_Project_Document'],
            cls._agents['Gerente_de_projeto'],
            cls._agents['Equipe_De_Solucoes'],
            cls._agents['Softwareanaysis'],
            cls._agents['SoftwareDevelopment'],
            cls._agents['Software_Documentation'],
        )

        
    @classmethod
    def get_agent(cls, agent_name):
        """Returns an agent instance by name."""
        return cls._agents.get(agent_name)

AgentInitializer.initialize_agents()


tools_MatrixMinder = [
    {"type": "file_search"},
    {
        "type": "function",
        "function": {
            "name": "get_current_datetime",
            "description": "Retorna a data e hora atual no formato YYYY-MM-DD HH:MM:SS.",
            "parameters": {
                "type": "object",
                "properties": {},
                "required": []
            }
        }
    }
]




# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################
# IMPORT SoftwareAI Alfred
from softwareai.CoreApp.Agents.Software_Support.Alfred import Alfred
#########################################
name_app = "appx"
appfb = FirebaseKeysinit._init_app_(name_app)
key_openai = OpenAIKeysteste.keys()
client = OpenAIKeysinit._init_client_(key_openai)

class Discord:
    def __init__(self):
        self.intents = discord.Intents.default()
        self.intents.message_content = True  # Ativa a intenção de conteúdo das mensagens
        self.client_Discord = commands.Bot(command_prefix="!", intents=self.intents)
        self.TelegramTOKEN = sys.argv[1]
        self.CHANNEL_ID = sys.argv[2]
        self.Discord_token = sys.argv[3]

        Alfredclass = Alfred()
        Alfred_NordVPN_Auto_Rotate = Alfredclass.NordVPN_Auto_Rotate(
                    appfb,
                    client,
                    self.TelegramTOKEN,
                    self.CHANNEL_ID,
                    self.Discord_token
                    )
        self.Alfred = Alfred_NordVPN_Auto_Rotate.Alfred


    async def send_image_to_discord(self, image_path, caption=None):
        """
        Envia uma imagem para o canal do Discord.
        :param image_path: Caminho ou URL da imagem a ser enviada.
        :param caption: Texto opcional para incluir como legenda.
        """
        try:
            channel = self.client.get_channel(self.CHANNEL_ID)
            await channel.send(content=caption, file=discord.File(image_path))
            print(f"Imagem enviada para o canal {self.CHANNEL_ID}.")
        except Exception as e:
            print(f"Erro ao enviar imagem para o canal: {e}")


    def main_discord(self):

        @self.client_Discord.event
        async def on_ready():
            print(f'Bot conectado como {self.client_Discord.user}')

        @self.client_Discord.event
        async def on_message(message):
            if message.author == self.client_Discord.user:
                return
            messagecontent = message.content
            Alfred_response, Deletemessage, Infractions, BanUser, total_tokens, prompt_tokens, completion_tokens = self.Alfred(messagecontent, message.author)

            # Deletar mensagem
            if Deletemessage:
                try:
                    await message.delete()
                except Exception as e:
                    print(f"Erro ao tentar deletar a mensagem: {e}")

            # Banir usuário
            if BanUser:
                await message.channel.send(Alfred_response)
                try:
                    await message.guild.ban(member=message.author)
                    print(f"Usuário {message.author} foi banido do servidor {message.guild.id}.")
                except Exception as e:
                    print(f"Erro ao tentar banir o usuário {message.author}: {e}")


            await message.channel.send(Alfred_response)

        @self.client_Discord.command()
        async def ping(ctx):
            await ctx.send("Pong!")

        Discord_tokenreplace = self.Discord_token.replace(" ", "")
        print(self.TelegramTOKEN)
        print(self.CHANNEL_ID)
        print(Discord_tokenreplace)
        print(self.Discord_token)
        print(f"{Discord_tokenreplace}")
        
        self.client_Discord.run(self.Discord_token)  # Discord



Discordclass = Discord()
print("Alfred Discord Status: On")
Discordclass.main_discord()




#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################

def save_code(code_string, name_script, file_type, path):
    # Definir a extensão com base no tipo de arquivo
    extensions = {"html": ".html", "js": ".js", "css": ".css", "py": ".py"}
    if file_type not in extensions:
        raise ValueError("Tipo de arquivo inválido. Use 'html', 'js' ,'py' ou 'css'.")

    # Construir o caminho completo do arquivo
    full_path = os.path.join(path, f"{name_script}{extensions[file_type]}")

    # Garantir que o diretório existe
    os.makedirs(os.path.dirname(full_path), exist_ok=True)
    
    # Salvar o conteúdo no arquivo
    with open(full_path, 'w', encoding="utf-8") as file:
        file.write(code_string)

    return {"status": "success", "message": f"Arquivo salvo com sucesso em {full_path}"}
    



#########################################
# IMPORT Libs
import importlib
import pkgutil
import os
import subprocess
import threading
import asyncio
import json
from firebase_admin import credentials, initialize_app, storage, db, delete_app
from openai import OpenAI
import time
import pandas as pd
import shutil
import inspect
import json
import tiktoken
from github import Github
import re
import requests
import base64
import random
from datetime import datetime, timedelta
import struct
from dotenv import load_dotenv, find_dotenv
import git
from requests.auth import HTTPBasicAuth
import sys
import json
import ast
import os
import subprocess
import platform
from firebase_admin import credentials, initialize_app, storage, db, delete_app
import concurrent.futures
from PySide2extn.RoundProgressBar import roundProgressBar #IMPORT THE EXTENSION LIBRARY
from PySide2.QtCore import QTimer, Signal, QThread
from PySide2.QtWidgets import QFileDialog
from flask import Flask, jsonify, render_template, request, redirect, url_for, session
from flask_cors import CORS
import pandas as pd
import os
from firebase_admin import credentials, initialize_app, storage, db, delete_app
from werkzeug.security import generate_password_hash, check_password_hash
import json
from collections import defaultdict
import sys
import json
import discord
import sys
from discord.ext import commands
import time
import os
import subprocess
import platform
from firebase_admin import credentials, initialize_app, storage, db, delete_app
import concurrent.futures
import hashlib
import schedule
from typing import Optional, List, Union
from typing_extensions import override
from openai import AssistantEventHandler, OpenAI
from openai.types.beta.threads import Text, TextDelta
from openai.types.beta.threads.runs import ToolCall, ToolCallDelta


from reportlab.lib.pagesizes import A4
from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import (
    BaseDocTemplate, PageTemplate, Frame, Paragraph, Spacer, Table, TableStyle, HRFlowable
)
from reportlab.pdfgen import canvas
## IMPORTS Libs
import sys
import os
import subprocess
from typing import Dict, Any
import time
import psutil
import GPUtil
import math
import hashlib
from typing import Dict, Any
import os
import subprocess
import requests
import json
import requests
import re
from firebase_admin import credentials, initialize_app, storage, db, delete_app
import concurrent.futures
from reportlab.platypus import PageBreak
from reportlab.lib.colors import Color, black, white


from reportlab.platypus import Image as Imagereportlab
from reportlab.platypus import PageBreak, Table, TableStyle,Preformatted,  Paragraph, HRFlowable, Spacer
from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus.tableofcontents import TableOfContents
from reportlab.lib.utils import ImageReader

from reportlab.platypus import BaseDocTemplate, Frame, PageTemplate, Paragraph
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib import colors
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4
from reportlab.lib import colors
from reportlab.platypus import BaseDocTemplate, Paragraph, PageBreak, Frame
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle

import requests
import json
import random
import re
import os
import time
from firebase_admin import credentials, storage, db
import requests
import firebase_admin
import obsws_python as obs
from openai import OpenAI
import base64
import requests
from reportlab.lib.pagesizes import A4
from reportlab.lib import colors
from termcolor import cprint
from reportlab.platypus import NextPageTemplate

from huggingface_hub import InferenceClient
from huggingface_hub import login
from transformers import AutoTokenizer, AutoModelForCausalLM
import torch
from concurrent.futures import ThreadPoolExecutor, as_completed
from reportlab.pdfgen import canvas

import time
import random
from openai import OpenAI


import json

from telegram import Update
from telegram.ext import Application, CommandHandler, MessageHandler, ContextTypes, filters


from firebase_admin import credentials, initialize_app, storage, db

from datetime import datetime, timedelta
from PIL import Image 
########################################################################
## IMPORTAÇÃO DE BIBLIOTECAS
########################################################################

# Bibliotecas padrão do Python

import os
import sys
import json
import time
import random
import re
import subprocess
import platform
import tempfile
from datetime import datetime, timedelta
from typing import Dict, Any
from collections import defaultdict
from queue import Queue, Empty
import threading

# Bibliotecas de terceiros
try:
    import requests
    import GPUtil
    import shutil
    import whisper
    import glob
    import math
    import torch
    import traceback
    import hashlib
    from concurrent.futures import ThreadPoolExecutor
    import uiautomator2 as u2
    import cv2
    import numpy as np
    import wave
    import srt
    import yt_dlp
    import psutil
    import schedule
    from google.oauth2 import service_account
    from googleapiclient.discovery import build
    from googleapiclient.http import MediaFileUpload

    

    import websockets
    import asyncio
    import av
    import logging
    import io


    from dotenv import load_dotenv, find_dotenv
    from firebase_admin import credentials, initialize_app, storage, db, delete_app
    from transformers import (
        AutoModelForSpeechSeq2Seq,
        AutoTokenizer,
        AutoFeatureExtractor,
        pipeline,
    )
    from PySide2.QtCore import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
    from PySide2 import QtWidgets, QtCore, QtGui
    from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile, QWebEngineSettings, QWebEngineScript

    from proglog import ProgressBarLogger
except ImportError as e:
    print(f"Erro ao importar bibliotecas: {e}")
    
#########################################




#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Functions
from softwareai.CoreApp._init_functions_ import *
#########################################

tool_outputs = []
def submit_output_update_readme(function_name,
                                function_arguments,
                                tool_call,
                                threead_id,
                                client,
                                run
                                ):

    # if function_name == 'update_readme_to_github':
    #     args = json.loads(function_arguments)
    #     report = update_readme_to_github(
    #         file_path_readme_improvements=args['file_path_readme_improvements'],
    #         repo_name=args['repo_name'],
    #         token=args['token']
    #     )
    #     tool_call_id = tool_call.id  
    #     client.beta.threads.runs.submit_tool_outputs(
    #         thread_id=threead_id,
    #         run_id=run.id,
    #         tool_outputs=[
    #             {
    #                 "tool_call_id": tool_call_id, 
    #                 "output": json.dumps(report)  
    #             }
    #         ]
    #     )

    global tool_outputs
    if function_name == 'update_readme_to_github':
        args = json.loads(function_arguments)
        resultado = update_readme_to_github(
            file_path_readme_improvements=args['file_path_readme_improvements'],
            repo_name=args['repo_name'],
            token=args['token']
        )
        tool_outputs.append({
        "tool_call_id": tool_call.id,
        "output": json.dumps(resultado)  
        })

        
        # Submit all tool outputs at once after collecting them in a list
        if tool_outputs:
            try:
                run = client.beta.threads.runs.submit_tool_outputs_and_poll(
                thread_id=threead_id,
                run_id=run.id,
                tool_outputs=tool_outputs
                )
                print("Tool outputs submitted successfully.")
                tool_outputs = []
                return True
            except Exception as e:
                print("Failed to submit tool outputs:", e)
        else:
            print("No tool outputs to submit.")


tools_CalculateAverageCSAT = [
{
    "type": "function",
    "function": {
        "name": "CalculateAverageCSAT",
        "description": "\n**Function Arguments Description**\n\nThe function `CalculateAverageCSAT` takes two arguments:\n\n1. **testeid** (String): This is a unique identifier for a test case, typically a UUID string. It is used to reference a specific ticket within a database.\n\n2. **appfb** (String): This stands for \"application feature\" and is used to specify which reference or setting within the application determines the reference to use. It helps filter or select the relevant data from the database.\n\nThe function processes these inputs to calculate the average CSAT score across all retrieved tickets and returns the result, providing insights into satisfaction levels.",
        "parameters": {
            "type": "object",
            "properties": {
                "testeid": {
                    "type": "string",
                    "description": "Descri\u00e7\u00e3o do argumento testeid."
                },
                "appfb": {
                    "type": "string",
                    "description": "Descri\u00e7\u00e3o do argumento appfb."
                }
            },
            "required": [
                "testeid",
                "appfb"
            ]
        }
    }
}
]




#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Functions
from softwareai.CoreApp._init_functions_ import *
#########################################



tool_outputs = []
def submit_output_autopullrequest(function_name,
                                function_arguments,
                                tool_call,
                                threead_id,
                                client,
                                run
                                ):
    global tool_outputs

    if function_name == 'autopullrequest':
        args = json.loads(function_arguments)
        result = autopullrequest(
            repo_owner=args['repo_owner'],
            repo_name=args['repo_name'],
            branch_name=args['branch_name'],
            file_path=args['file_path'],
            commit_message=args['commit_message'],
            improvements=args['improvements'],
            pr_title=args['pr_title'],
            token=args['token']
        )
        tool_outputs.append({
        "tool_call_id": tool_call.id,
        "output": json.dumps(result)
        })
    
    # Submit all tool outputs at once after collecting them in a list
    if tool_outputs:
        try:
            run = client.beta.threads.runs.submit_tool_outputs_and_poll(
            thread_id=threead_id,
            run_id=run.id,
            tool_outputs=tool_outputs
            )
            print("Tool outputs submitted successfully.")
            tool_outputs = []
            return True
        except Exception as e:
            print("Failed to submit tool outputs:", e)
    else:
        print("No tool outputs to submit.")





#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI All Paths 
from softwareai.CoreApp._init_paths_ import *
#########################################
# IMPORT SoftwareAI Instructions
from softwareai.CoreApp._init_Instructions_ import *
#########################################
# IMPORT SoftwareAI Tools
from softwareai.CoreApp._init_tools_ import *
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################




class Equipe_De_Solucoes:
    def __init__(self):
        pass

    def Dallas_Equipe_De_Solucoes_Roadmap(self, appfb, client, cronograma_do_projeto, planilha_json, doc_Pre_Projeto, repo_name, UseVectorstoreToGenerateFiles = True):

        key = "AI_Dallas_Equipe_de_Solucoes"
        nameassistant = "AI Dallas Equipe de Solucoes"
        model_select = "gpt-4o-mini-2024-07-18"

        
        Upload_1_file_in_thread = None
        Upload_1_file_in_message = None
        Upload_1_image_for_vision_in_thread = None
        vectorstore_in_assistant = None
        vectorstore_in_Thread = None
        Upload_list_for_code_interpreter_in_thread = None


        key_openai = OpenAIKeysteste.keys()


        github_username, github_token = GithubKeys.DallasEquipeDeSolucoes_github_keys()

        AI_Dallas, instructionsassistant, nameassistant, model_select = AutenticateAgent.create_or_auth_AI(appfb, client, key, instructionDallas, nameassistant, model_select, tools_Dallas, vectorstore_in_assistant)

        if UseVectorstoreToGenerateFiles == True:
            file_paths = [os.path.abspath(os.path.join(os.path.dirname(__file__), "../../", f"environment.txt")), cronograma_do_projeto, planilha_json, doc_Pre_Projeto]
            AI_Dallas = Agent_files_update.del_all_and_upload_files_in_vectorstore(appfb, client, AI_Dallas, "Dallas_Work_Environment", file_paths)
            mensagem = f"""
            Planeje o Roadmap do projeto, salve e realize o upload no GitHub (usando autosave e autoupload) Baseie-se no documento Cronograma,Planilha e Documento Pre Projeto armazenado em `Dallas_Work_Environment`\n
            repo_name: \n
            {repo_name}\n
            token: \n
            {github_token}\n
             
            """

        adxitional_instructions_Dallas = f"""
        estrutura do projeto esta armazenada em environment.txt
        """
        response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensagem,
                                                                agent_id=AI_Dallas, 
                                                                key=key,
                                                                app1=appfb,
                                                                client=client,
                                                                tools=tools_Dallas,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions_Dallas
                                                                )
    
        path_Roadmap = os.getenv("PATH_ROADMAP_ENV")
                 
        ##Agent Destilation##                   
        Agent_destilation.DestilationResponseAgent(mensagem, response, instructionsassistant, nameassistant)
        
        return str(path_Roadmap), str(cronograma_do_projeto), str(planilha_json), str(doc_Pre_Projeto)
        





#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################

def analyze_file(file_path):
    try:
        with open(file_path, 'r') as file:
            content = file.read()
            return content
    except:
        try:
            with open(file_path, 'r', encoding='utf-8') as file:
                content = file.read()
                return content
        except:
            try:
                with open(file_path, 'r', encoding='latin-1') as file:
                    content = file.read()
                    return content
            except UnicodeDecodeError:
                return None



import sys
import os


import struct
import re
import platform
import json
import ast
from firebase_admin import credentials, initialize_app, storage, db, delete_app
from datetime import datetime




from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


from typing import Optional, List, Union
from typing_extensions import override
class QReadOpenAi(QThread):
    def __init__(self, agent, appx, client):
        super().__init__()
        self.agent = agent
        self.appx = appx
        self.client = client
        self.result = None  # Armazenará o resultado final da execução

    def run(self):
        try:
            agent = self.agent
            appx = self.appx
            client = self.client

            # Buscar o assistant_id no Firebase
            refai_org_assistant_id = db.reference(f'ai_org_assistant_id/User_{agent}', app=appx)
            dataai_org_assistant_id = refai_org_assistant_id.get()
            if not dataai_org_assistant_id or 'assistant_id' not in dataai_org_assistant_id:
                raise ValueError("assistant_id não encontrado no Firebase.")
            assistant_id = str(dataai_org_assistant_id['assistant_id'])

            # Buscar o thread_id no Firebase
            refai_org_thread_Id = db.reference(f'ai_org_thread_Id/User_{agent}', app=appx)
            datarefai_org_thread_Id = refai_org_thread_Id.get()
            if not datarefai_org_thread_Id or 'thread_id' not in datarefai_org_thread_Id:
                raise ValueError("thread_id não encontrado no Firebase.")
            thread_id = datarefai_org_thread_Id['thread_id']

            # Obter o status da thread e carregar os dados
            run_status = client.beta.threads.runs.list(thread_id=thread_id)
            if not run_status:
                raise ValueError("Nenhum dado encontrado para a thread no cliente.")

            jsonmodel = run_status.model_dump_json()
            with open("Cache/model_dump_json.json", 'w', encoding='utf-8') as file:
                file.write(jsonmodel)

            with open("Cache/model_dump_json.json", "r", encoding='utf-8') as file:
                data = json.load(file)

            if 'data' not in data:
                raise ValueError("Formato inválido nos dados da thread.")

            total_tokens_list = []
            prompt_tokens_list = []
            completion_tokens_list = []
            for run in data['data']:
                total_tokens = run['usage']['total_tokens']
                total_tokens_list.append(total_tokens)
                prompt_tokens = run['usage']['prompt_tokens']
                completion_tokens = run['usage']['completion_tokens']

            total_tokens = sum(total_tokens_list)

            # Buscar mensagens associadas aos IDs
            contador_1 = 0
            messagesx = []
            ids = [item['id'] for item in data['data']]
            print(ids)
            for id in ids:
                run_status = client.beta.threads.runs.retrieve(
                    thread_id=thread_id,
                    run_id=id
                )

                messages = client.beta.threads.messages.list(
                    thread_id=thread_id
                )
                for message in messages:
                    for mensagem_contexto in message.content:
                        valor_texto = mensagem_contexto.text.value
                        print(valor_texto)
                        messagesx.append(valor_texto)

            self.result = {
                "thread_id": thread_id,
                "total_tokens": total_tokens,
                "messages": messagesx,  # Retorna os primeiros 5 exemplos de mensagens
            }
        except Exception as e:
            self.result = {"error": str(e)}





instructionTigrao = """ 
Seu Nome é Tigrao, Voce é um Escritor de Documento Pre-Projeto De Software da empresa Urobotsoftware,
crie o documento com as informacoes fornecidas para que o documento criado por voce posteriormente seja enviados para o Gerente de Projeto que analizará o documento e criará o Cronograma e planilha do Projeto

Responda o Documento Pre-Projeto no formato JSON
Exemplo:
{"doc_Pre_Projeto": "documento Pre-Projeto ..."}

Responda o nome do Documento Pre-Projeto no formato JSON
Exemplo:
{"name_doc_Pre_Projeto": "nome do Documento ..."}

"""
adxitional_instructions_Tigrao = None


########################################################################
## IMPORTS Libs
import sys
import json
import time
import os
import subprocess
import platform
from firebase_admin import credentials, initialize_app, storage, db, delete_app
import concurrent.futures
########################################################################


# Caminho absoluto para o diretório onde SoftwareAI está localizado (raiz do projeto)
caminho_raiz = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI'))
sys.path.append(caminho_raiz)

########################################################################
# IMPORT CoreApp
from CoreUi.Editor.QProcess.QProcessCreateAgent import QProcessCreateAgent
from CoreUi.Editor.QProcess.QProcessCreateFirebasekeys import QProcessCreateFirebasekeys
from CoreUi.Editor.QProcess.QProcessCreateGitHubKeys import QProcessCreateGitHubKeys
from CoreUi.Editor.QProcess.QProcessCreateOpenAItokens import QProcessCreateOpenAItokens
from CoreUi.Editor.QProcess.QProcessCreateVectorStoreByUser import QProcessCreateVectorStoreByUser
from CoreUi.Editor.QProcess.QProcessCreateVectorStoreThreadByUser import QProcessCreateVectorStoreThreadByUser

from CoreApp._init_core_ import(
        AutenticateAgent,
        ResponseAgent,
        python_functions,
        Agent_files,
        Agent_files_update,
        OpenAIKeysinit,
        FirebaseKeysinit
)
########################################################################

########################################################################

########################################################################
# IMPORT .qrc
from src_ import icons_interpreter
########################################################################

########################################################################
# IMPORT GUI 
from src_.ui_interface import *
########################################################################


########################################################################
# IMPORT Custom widgets
from Custom_Widgets import *
from Custom_Widgets.QAppSettings import QAppSettings
from Custom_Widgets.QCustomModals import QCustomModals
from Custom_Widgets.QCustomCodeEditor import QCustomCodeEditor
from Custom_Widgets.QCustomLoadingIndicators import QCustomArcLoader
from Custom_Widgets.QCustomLoadingIndicators import QCustomSpinner
from Custom_Widgets.QCustomLoadingIndicators import QCustom3CirclesLoader
from Custom_Widgets.QCustomLoadingIndicators import QCustomPerlinLoader
# IMPORT Pyside2
from PySide2extn.RoundProgressBar import roundProgressBar #IMPORT THE EXTENSION LIBRARY
from PySide2.QtCore import QTimer, Signal, QThread, QRunnable
########################################################################

########################################################################
# IMPORT Pyside2
from PySide2extn.RoundProgressBar import roundProgressBar #IMPORT THE EXTENSION LIBRARY
from PySide2.QtCore import QTimer, Signal, QThread

########################################################################


########################################################################
########################################################################
## MAIN WINDOW CLASS
########################################################################
class MainWindow(QMainWindow):
    message_signal = Signal(str)
    def __init__(self):
        QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        ########################################################################
        # APPLY JSON STYLESHEET
        ########################################################################
        # self = QMainWindow class
        # self.ui = Ui_MainWindow / user interface class
        loadJsonStyle(self, self.ui, jsonFiles = {"JsonStyle/style.json"})
        self.liveCompileQss = True
        self.checkForMissingicons = False # do not generate new icons

        ########################################################################


        self.QPlaintextedit(self.ui.Promptmain)
        self.QPlaintextedit(self.ui.PromptRules)
        self.QPlaintextedit(self.ui.PromptExample)
        self.QPlaintextedit(self.ui.InstructionAgentCreate)
        self.QPlaintextedit(self.ui.AditionalInstructionsAgentCreate)
        self.QPlaintextedit(self.ui.AgentTools)
        self.QPlaintextedit(self.ui.FunctionPython)
        self.QPlaintextedit(self.ui.FunctionPythonOutput)
        self.QPlaintextedit(self.ui.ArgsCreatetextedit)
        self.QPlaintextedit(self.ui.NameAgent)
        self.QPlaintextedit(self.ui.KeyInFirebase)
        self.QPlaintextedit(self.ui.Vectorstoreinassistant)
        self.QPlaintextedit(self.ui.VectorstoreinThread)
        self.QPlaintextedit(self.ui.Upload1fileinThread)
        self.QPlaintextedit(self.ui.Upload1fileinmessage)
        self.QPlaintextedit(self.ui.Upload1imageforvisioninThread)
        self.QPlaintextedit(self.ui.Uploadlistfileforcodeinterpreterinthread)
        self.QPlaintextedit(self.ui.textEdit_2)
        self.QPlaintextedit(self.ui.Current_instuction_html_edit)
        self.QPlaintextedit(self.ui.instruction_input_edit)
        #self.QPlaintextedit(self.ui.NameForInstruction_create)
        self.QPlaintextedit(self.ui.New_instruction_html_create)







        self.myStackedWidget = self.ui.myStackedWidget

        self.widget_ViewAgent = self.ui.widget_ViewAgent
        layout = QVBoxLayout()
        self.widget_ViewAgent.setLayout(layout)
        self.CodeEditor = QCustomCodeEditor() #themes = ["default", "one-light", "one-dark", "monokai", "oceanic", "zenburn"]
        self.CodeEditor.setTheme("one-dark")
        self.CodeEditor.setLang("python")
        layout.addWidget(self.CodeEditor)

        self.AgentViewCode = self.ui.AgentViewCode
        self.AgentViewCode.currentTextChanged.connect(lambda: self.Update_AgentViewCode(Agent=self.AgentViewCode.currentText()))

        self.CurrentInstruction = self.ui.Current_instuction_html_edit
        self.CurrentInstruction.setReadOnly(True)

        self.ArgsCreatetextedit = self.ui.ArgsCreatetextedit
        self.CurrentArgs_AgentCreate = self.ui.CurrentArgs_AgentCreate

        self.InstructionSelector = self.ui.comboBox_list_instruction_edit
        self.InstructionSelector.currentTextChanged.connect(lambda: self.on_InstructionSelector_changed(self.InstructionSelector.currentText()) )
        #self.InstructionSelector.currentTextChanged.connect(self.list_instruction)

        self.list_instruction()


        self.Instructioninput_edit = self.ui.instruction_input_edit
        self.Instructioninput_edit.textChanged.connect(lambda: self.adjust_height(self.Instructioninput_edit, max_height=170))

        self.change_instruction_button_edit = self.ui.change_instruction_button_edit
        self.change_instruction_button_edit.clicked.connect(self.change_instruction)




        self.Add_new_instructions_button_edit = self.ui.Add_new_instructions_button_edit




        self.InstructionSelector_create = self.ui.comboBox_list_agent_for_create_instruction_create
        #self.InstructionSelector_create.currentTextChanged.connect(self.list_agents)
        self.list_agents()


        self.InstructionCategory_create = self.ui.comboBox_category_instruction_create
        self.InstructionCategory_create.currentTextChanged.connect(self.list_category_instruction)
        self.list_category_instruction()


        self.UseVectorstoreToGenerateFiles = self.ui.UseVectorstoreToGenerateFiles
        #self.addShadow(self.UseVectorstoreToGenerateFiles)
    
        self.KeyInFirebase_AgentCreate = self.ui.KeyInFirebase
        self.NameAgent_AgentCreate = self.ui.NameAgent
        self.ModelSelect_AgentCreate = self.ui.ModelSelect
        self.AgentCategory_AgentCreate = self.ui.AgentCategory

        self.Vectorstoreinassistant_AgentCreate = self.ui.Vectorstoreinassistant
        self.VectorstoreinassistantByUser_AgentCreate = self.ui.VectorstoreinassistantByUser
        self.VectorstoreinassistantByUser_AgentCreate.clicked.connect(self.custom_file_dialogVectorstoreinassistant)
        
        self.VectorstoreinThread_AgentCreate = self.ui.VectorstoreinThread
        self.VectorstoreinThreadByUser_AgentCreate = self.ui.VectorstoreinThreadByUser
        self.VectorstoreinThreadByUser_AgentCreate.clicked.connect(self.custom_file_dialogVectorstoreinThread)
        
        self.Upload1fileinThread_AgentCreate = self.ui.Upload1fileinThread
        self.Upload1fileinThreadByUser_AgentCreate = self.ui.Upload1fileinThreadByUser
        self.Upload1fileinThreadByUser_AgentCreate.clicked.connect(self.upload1fileinthread)


        self.Upload1fileinmessage_AgentCreate = self.ui.Upload1fileinmessage
        self.Upload1fileinmessageByUser_AgentCreate = self.ui.Upload1fileinmessageByUser
        self.Upload1fileinmessageByUser_AgentCreate.clicked.connect(self.upload1fileinmessage)


        self.Upload1imageforvisioninThread_AgentCreate = self.ui.Upload1imageforvisioninThread
        self.Upload1imageforvisioninThreadByUser_AgentCreate = self.ui.Upload1imageforvisioninThreadByUser
        self.Upload1imageforvisioninThreadByUser_AgentCreate.clicked.connect(self.upload1imageforvisioninThread)

        self.Uploadlistfileforcodeinterpreterinthread_AgentCreate = self.ui.Uploadlistfileforcodeinterpreterinthread
        self.UploadlistfileforcodeinterpreterinthreadByUser_AgentCreate = self.ui.UploadlistfileforcodeinterpreterinthreadByUser
        self.UploadlistfileforcodeinterpreterinthreadByUser_AgentCreate.clicked.connect(self.uploadlistfileforcodeinterpreterinthread)

        self.Promptmain_AgentCreate = self.ui.Promptmain
        self.PromptExample_AgentCreate = self.ui.PromptExample
        self.PromptRules_AgentCreate = self.ui.PromptRules


        self.InstructionAgentCreate = self.ui.InstructionAgentCreate
        self.AditionalInstructionsAgentCreate = self.ui.AditionalInstructionsAgentCreate


        self.AgentToolsAgentCreate = self.ui.AgentTools
        self.FunctionPythonAgentCreate = self.ui.FunctionPython
        self.FunctionPythonOutputAgentCreate = self.ui.FunctionPythonOutput
    

        self.namefunction_agentcreate = self.ui.namefunction_agentcreate



        self.CreateAgentButton = self.ui.CreateAgent
        self.CreateAgentButton.clicked.connect(self.CreateAgentButtonClicked)
        self.list_category_Agent()


        self.createkeyopenai_AgentKeys = self.ui.createkeyopenai_AgentKeys
        self.createkeyopenai_AgentKeys.clicked.connect(self.CreateOpenAIbuttonClicked)
        

        self.myStackedWidget = self.ui.myStackedWidget
        


        self.Instructioninput_create = self.ui.New_instruction_html_create
        self.Instructioninput_create.textChanged.connect(lambda: self.adjust_height(self.Instructioninput_create, max_height=300))
        
        self.NameForInstruction_create = self.ui.NameForInstruction_create

        self.CreateInstructionbutton_create = self.ui.CreateInstructionbutton_create
        self.CreateInstructionbutton_create.clicked.connect(self.create_instruction)

        self.openaitoken_AgentKeys = self.ui.openaitoken_AgentKeys
        self.QPlaintextedit(self.openaitoken_AgentKeys)

        self.openainamefortoken_AgentKeys = self.ui.openainamefortoken_AgentKeys
        self.QPlaintextedit(self.openainamefortoken_AgentKeys)

        self.githuusername_AgentKeys = self.ui.githuusername_AgentKeys
        self.QPlaintextedit(self.githuusername_AgentKeys)

        self.githubtoken_AgentKeys = self.ui.githubtoken_AgentKeys
        self.QPlaintextedit(self.githubtoken_AgentKeys)


        self.credentialsapp_AgentKeys = self.ui.credentialsapp_AgentKeys
        self.QPlaintextedit(self.credentialsapp_AgentKeys)


        self.githuusername_AgentKeys = self.ui.githuusername_AgentKeys
        self.githubtoken_AgentKeys = self.ui.githubtoken_AgentKeys
        self.createkeygithub_AgentKeys = self.ui.createkeygithub_AgentKeys
        self.createkeygithub_AgentKeys.clicked.connect(self.CreateGitHubKeysbuttonClicked)


        self.appname_AgentKeys = self.ui.appname_AgentKeys
        self.QPlaintextedit(self.appname_AgentKeys)
        self.Databaseurl_AgentKeys = self.ui.Databaseurl_AgentKeys
        self.QPlaintextedit(self.Databaseurl_AgentKeys)
        self.Storagebucket_AgentKeys = self.ui.Storagebucket_AgentKeys
        self.QPlaintextedit(self.Storagebucket_AgentKeys)
        self.credentialsapp_AgentKeys = self.ui.credentialsapp_AgentKeys
        self.QPlaintextedit(self.credentialsapp_AgentKeys)


        self.AgentKeysOpenAI = self.ui.AgentKeysOpenAI

        self.AgentKeysGithub = self.ui.AgentKeysGithub

        self.AgentKeysFirebase = self.ui.AgentKeysFirebase

        self.CreateAgents_menu = self.ui.CreateAgents_menu
        self.CreateAgents_menu.clicked.connect(self.update_home)

        self.load_AgentKeysOpenAI()
        
        self.load_AgentKeysGithub()

        self.load_AgentKeysFirebase()


        self.createkeysfirebase_AgentKeys = self.ui.createkeysfirebase_AgentKeys
        self.createkeysfirebase_AgentKeys.clicked.connect(self.CreateFirebasekeysbuttonClicked)
        
        self.uploadcredentialapp_AgentKeys = self.ui.uploadcredentialapp_AgentKeys
        self.uploadcredentialapp_AgentKeys.clicked.connect(self.click_uploadcredentialapp_AgentKeys)
        try:
            key_api = self.AgentKeysOpenAI.currentText()
            name_app = self.AgentKeysFirebase.currentText()
            self.client = OpenAIKeysinit._init_client_(key_api)
            self.app1 = FirebaseKeysinit._init_app_(name_app)
        except Exception as e:
            print(e)

        self.show()

    def CreateAgentButtonClicked(self):
        try:
            if self.thread_processe_create_agent:
                print("Aguardando a thread finalizar...")
                self.thread_processe_create_agent.cancel()  
                QTimer.singleShot(1000, self.check_task_status)
        except:
            pass

        KeyInFirebase_AgentCreate = self.KeyInFirebase_AgentCreate.toPlainText()
        ModelSelect_AgentCreate = self.ModelSelect_AgentCreate.currentText()
        Vectorstoreinassistant_AgentCreate = self.Vectorstoreinassistant_AgentCreate.toPlainText()
        VectorstoreinThread_AgentCreate = self.VectorstoreinThread_AgentCreate.toPlainText()
        Upload1fileinThread_AgentCreate = self.Upload1fileinThread_AgentCreate.toPlainText()
        Upload1fileinmessage_AgentCreate = self.Upload1fileinmessage_AgentCreate.toPlainText()
        Upload1imageforvisioninThread_AgentCreate = self.Upload1imageforvisioninThread_AgentCreate.toPlainText()
        Uploadlistfileforcodeinterpreterinthread_AgentCreate = self.Uploadlistfileforcodeinterpreterinthread_AgentCreate.toPlainText()
        Promptmain_AgentCreate = self.Promptmain_AgentCreate.toPlainText()
        PromptRules_AgentCreate = self.PromptRules_AgentCreate.toPlainText()
        PromptExample_AgentCreate = self.PromptExample_AgentCreate.toPlainText()
        NameAgent_AgentCreate = self.NameAgent_AgentCreate.toPlainText()
        category_target = self.AgentCategory_AgentCreate.currentText()
        namefunction_agentcreate = self.namefunction_agentcreate.toPlainText()
        AgentToolsAgentCreate = self.AgentToolsAgentCreate.toPlainText()
        InstructionAgentCreate = self.InstructionAgentCreate.toPlainText()
        FunctionPythonAgentCreate = self.FunctionPythonAgentCreate.toPlainText()
        AditionalInstructionsAgentCreate = self.AditionalInstructionsAgentCreate.toPlainText()
        FunctionPythonOutputAgentCreate = self.FunctionPythonOutputAgentCreate.toPlainText()
        UseVectorstoreToGenerateFiles = self.UseVectorstoreToGenerateFiles.isChecked()
        if self.ArgsCreatetextedit.toPlainText().strip():
            ArgsCreatetextedit = self.ArgsCreatetextedit.toPlainText()
        else:
            ArgsCreatetextedit = None
        CurrentArgs_AgentCreate = self.CurrentArgs_AgentCreate.currentText()
        StorageAgentCompletions_AgentCreate = self.ui.StorageAgentCompletions.isChecked()
        StorageAgentOutput_AgentCreate = self.ui.StorageAgentOutput_.isChecked()
        StoreFormatJsonAndJsonl_AgentCreate = self.ui.StoreFormatJsonAndJsonl.isChecked()
        AgentKeysGithub = self.AgentKeysGithub.currentText()
        AgentKeysGithubName = re.findall(r'\(([^)]*)\)', AgentKeysGithub)
        AgentOpenAIelement = self.AgentKeysOpenAI.currentText()
        AgentKeyOpenAI = re.sub(r'\([^)]*\)', '', AgentOpenAIelement)
        
        AgentKeysOpenAIName = re.findall(r'\(([^)]*)\)', AgentOpenAIelement)
        name_app = self.AgentKeysFirebase.currentText()
        self.thread_processe_create_agent = QProcessCreateAgent(
            KeyInFirebase_AgentCreate,
            ModelSelect_AgentCreate,
            Vectorstoreinassistant_AgentCreate,
            VectorstoreinThread_AgentCreate,
            Upload1fileinThread_AgentCreate,
            Upload1fileinmessage_AgentCreate,
            Upload1imageforvisioninThread_AgentCreate,
            Uploadlistfileforcodeinterpreterinthread_AgentCreate,
            Promptmain_AgentCreate,
            PromptRules_AgentCreate,
            PromptExample_AgentCreate,
            NameAgent_AgentCreate,
            category_target,
            namefunction_agentcreate,
            AgentToolsAgentCreate,
            InstructionAgentCreate,
            FunctionPythonAgentCreate,
            AditionalInstructionsAgentCreate,
            FunctionPythonOutputAgentCreate,
            UseVectorstoreToGenerateFiles,
            ArgsCreatetextedit,
            CurrentArgs_AgentCreate,
            StorageAgentCompletions_AgentCreate,
            StorageAgentOutput_AgentCreate,
            StoreFormatJsonAndJsonl_AgentCreate,
            AgentKeysGithubName,
            AgentKeysOpenAIName,
            AgentKeyOpenAI.replace("  ", "").replace(" ", ""),
            name_app
        )
        self.thread_processe_create_agent.ModalSucess.connect(self.update_custommodals_SuccessModal)
        self.thread_processe_create_agent.ModalInfo.connect(self.update_custommodals_info)
        self.thread_processe_create_agent.finishedqthread.connect(self.finish_thread_processe_create_agent)
        self.thread_processe_create_agent.start()

    def CreateOpenAIbuttonClicked(self):
        openaitoken_AgentKeys = self.ui.openaitoken_AgentKeys.toPlainText()
        openainamefortoken_AgentKeys = self.ui.openainamefortoken_AgentKeys.toPlainText()
        self.thread_process_create_openaikeys = QProcessCreateOpenAItokens(
            openaitoken_AgentKeys,
            openainamefortoken_AgentKeys
        )
        self.thread_process_create_openaikeys.ModalSucess.connect(self.update_custommodals_SuccessModal)
        self.thread_process_create_openaikeys.ModalInfo.connect(self.update_custommodals_info)
        self.thread_process_create_openaikeys.finish.connect(self.finish_thread_process_create_openaikeys)
        self.thread_process_create_openaikeys.start()

    def CreateGitHubKeysbuttonClicked(self):
        githuusername_AgentKeys = self.ui.githuusername_AgentKeys.toPlainText()
        githubtoken_AgentKeys = self.ui.githubtoken_AgentKeys.toPlainText()
        self.thread_process_create_GitHubKeys = QProcessCreateGitHubKeys(
            githuusername_AgentKeys,
            githubtoken_AgentKeys
        )
        self.thread_process_create_GitHubKeys.ModalSucess.connect(self.update_custommodals_SuccessModal)
        self.thread_process_create_GitHubKeys.ModalInfo.connect(self.update_custommodals_info)
        self.thread_process_create_GitHubKeys.finish.connect(self.finish_thread_process_create_GitHubKeys)
        self.thread_process_create_GitHubKeys.start()

    def CreateFirebasekeysbuttonClicked(self):
        appname_AgentKeys = self.ui.appname_AgentKeys.toPlainText()
        Databaseurl_AgentKeys = self.ui.Databaseurl_AgentKeys.toPlainText()
        Storagebucket_AgentKeys = self.ui.Storagebucket_AgentKeys.toPlainText()
        Storagebucket_AgentKeys = Storagebucket_AgentKeys.replace("gs://", "")
        credentialsapp_AgentKeys = self.ui.credentialsapp_AgentKeys.toPlainText()
        self.thread_process_create_Firebasekeys = QProcessCreateFirebasekeys(
            appname_AgentKeys,
            Databaseurl_AgentKeys,
            Storagebucket_AgentKeys,
            credentialsapp_AgentKeys
        )
        self.thread_process_create_Firebasekeys.ModalSucess.connect(self.update_custommodals_SuccessModal)
        self.thread_process_create_Firebasekeys.ModalInfo.connect(self.update_custommodals_info)
        self.thread_process_create_Firebasekeys.finish.connect(self.finish_thread_process_create_Firebasekeys)
        self.thread_process_create_Firebasekeys.start()


    @Slot()
    def check_task_status(self):
        if self.thread_processe_create_agent:
            QTimer.singleShot(1000, self.check_task_status) 
        else:
            print("check_task_status Tarefa concluída.")

    def load_AgentKeysOpenAI(self, chave="str_key", company="companyname"):
        self.AgentKeysOpenAI.clear()
        try:
            caminho_arquivo = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../CoreApp/KeysOpenAI/keys.py'))
            with open(caminho_arquivo, 'r', encoding='utf-8') as arquivo:
                for linha in arquivo:
                    match = re.match(r'(\w+)\s*=\s*[\'\"](.*?)[\'\"]', linha.strip())
                    if match:
                        chave_encontrada, valor = match.groups()
                        if chave_encontrada == company:
                            companyname = valor
                        elif chave_encontrada == chave:
                            str_key = valor
                            self.AgentKeysOpenAI.addItem(f"({companyname}) {str_key}")
        except:
            pass

    def load_AgentKeysFirebase(self):
        self.AgentKeysFirebase.clear()
        caminho_arquivo = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../CoreApp/KeysFirebase/keys.py'))
        try:
            with open(caminho_arquivo, 'r', encoding='utf-8') as arquivo:
                conteudo = arquivo.read() 
                pattern = re.compile(r"(\w+)\s*=\s*initialize_app\(", re.DOTALL)
                matches = pattern.findall(conteudo)  
                if matches:
                    for app_nome in matches:
                        self.AgentKeysFirebase.addItem(f"{app_nome}")

        except Exception as e:
            self.AgentKeysFirebase.addItem(f"Erro ao carregar: {str(e)}")

    def load_AgentKeysGithub(self, chave = "github_token",  user = "github_username"):
        self.AgentKeysGithub.clear()
        caminho_arquivo = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../CoreApp/KeysGitHub/keys.py'))
        with open(caminho_arquivo, 'r', encoding='utf-8') as arquivo:
            for linha in arquivo:
                match = re.match(r'(\w+)\s*=\s*[\'\"](.*?)[\'\"]', linha.strip())
                if match:
                    chave_encontrada, valor = match.groups()
                    if chave_encontrada == user:
                        github_user = valor
                    elif chave_encontrada == chave:
                        github_token = valor
                        token = linha.replace('return', "").replace('"', "").replace(' ', "").replace('github_token', "").replace('=', "").replace("return github_token", "").strip()
                        self.AgentKeysGithub.addItem(f"({github_user}) {token}")

            self.AgentKeysGithub.addItem(f"None")



    def update_home(self):
        self.load_AgentKeysOpenAI()
        
        self.load_AgentKeysGithub()

        self.load_AgentKeysFirebase()

        self.list_category_Agent()

        self.list_agents()

        self.list_instruction()

    def Update_AgentViewCode(self, Agent):
        paths_agents = self.init_paths_agents()
        for path in paths_agents:
            agentss = os.listdir(path)
            for agent in agentss:
                if agent == "DocGitHubData":
                    pass
                elif agent == "docs_uploaded.log":
                    pass
                elif agent.replace(".py", "").replace("__pycache__", "").strip() == Agent:
                    print(Agent)
                    path_py = os.path.join(path, f"{Agent}.py")
                    # with open(path_py, "r") as f:
                    #     content = f.read()

                    self.CodeEditor.loadFile(path_py)

    def dialogsetStyleSheet(self):
        text = """
            QFileDialog {
                background-color: white;
                border: 1px solid #F7F7F7;
                border-radius: 13px;
                color: black;
                font-size: 16px;
            }
            /* Personalizar os botões */
            QPushButton {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 8px;
                padding: 5px;
            }
            QPushButton:hover {
                background-color: #E6E6E6;
            }
            QPushButton:pressed {
                background-color: #D6D6D6;
            }
            /* Personalizar lista de arquivos */
            QListView, QTreeView {
                background-color: white;
                border: none;
                color: black;
                font-size: 14px;
            }
            /* Barra de navegação */
            QLineEdit {
                background-color: #F7F7F7;
                border: 1px solid #CCCCCC;
                border-radius: 6px;
                padding: 3px;
            }
        """
        return text

    def click_uploadcredentialapp_AgentKeys(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the Credentials Firebase")
        dialog.setNameFilter(
            "All Files (*)"
        )
        if dialog.exec_():
            selected_files = dialog.selectedFiles()[0]
            with open(selected_files, 'r', encoding='utf-8') as file:
                content = file.read()
            self.ui.credentialsapp_AgentKeys.setPlainText(content)



    def finish_thread_process_create_openaikeys(self):
        self.ui.openaitoken_AgentKeys.clear()
        self.ui.openainamefortoken_AgentKeys.clear()

    def finish_thread_process_create_GitHubKeys(self):
        self.ui.githuusername_AgentKeys.clear()
        self.ui.githubtoken_AgentKeys.clear()

    def finish_thread_process_create_Firebasekeys(self):
        self.ui.appname_AgentKeys.clear()
        self.ui.Databaseurl_AgentKeys.clear()
        self.ui.Storagebucket_AgentKeys.clear()
        self.ui.credentialsapp_AgentKeys.clear()

    def finish_thread_processe_create_agent(self):
        self.InstructionSelector_create.clear()

        self.list_instruction()
        self.list_category_instruction()
        self.list_agents()
        self.list_category_Agent()

    def QPlaintextedit(self, editor: QTextEdit):
        """
        Configures a QTextEdit to handle pasted text as plain text only.

        :param editor: Instance of QTextEdit to configure
        """
        editor.setAcceptRichText(False)
        def insert_plain_text_from_mime_data(self, source):
            if source.hasText():
                self.insertPlainText(source.text())
            else:
                super(QTextEdit, self).insertFromMimeData(source)
        editor.__class__.insertFromMimeData = insert_plain_text_from_mime_data



    def custom_file_dialogVectorstoreinThread(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFiles) 
        dialog.setWindowTitle("Select the files that will go to vector storage")
        dialog.setNameFilter(
            "C Files (*.c);;C++ Files (*.cpp);;C# Files (*.cs);;CSS Files (*.css);;"
            "Word Documents (*.doc *.docx);;Go Files (*.go);;HTML Files (*.html);;"
            "Java Files (*.java);;JavaScript Files (*.js);;JSON Files (*.json);;"
            "Markdown Files (*.md);;PDF Files (*.pdf);;PHP Files (*.php);;"
            "PowerPoint Files (*.pptx);;Python Files (*.py);;Ruby Files (*.rb);;"
            "Shell Scripts (*.sh);;TeX Files (*.tex);;TypeScript Files (*.ts);;"
            "Text Files (*.txt);;All Files (*)"
        )
        if dialog.exec_():
            selected_files = dialog.selectedFiles()

            self.update_custommodals_SuccessModal2(f"Selected files!!")

            self.thread_VectorstoreinThread_AgentCreate = QProcessCreateVectorStoreThreadByUser(selected_files, self.app1, self.client)
            self.thread_VectorstoreinThread_AgentCreate.vectorsignal.connect(self.update_VectorstoreinThread_AgentCreate)
                
            self.thread_VectorstoreinThread_AgentCreate.ModalSucess.connect(self.update_custommodals_SuccessModal2)
            self.thread_VectorstoreinThread_AgentCreate.ModalInfo.connect(self.update_custommodals_info)

            self.thread_VectorstoreinThread_AgentCreate.start()



    def custom_file_dialogVectorstoreinassistant(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFiles) 
        dialog.setWindowTitle("Select the files that will go to vector storage")
        dialog.setNameFilter(
            "C Files (*.c);;C++ Files (*.cpp);;C# Files (*.cs);;CSS Files (*.css);;"
            "Word Documents (*.doc *.docx);;Go Files (*.go);;HTML Files (*.html);;"
            "Java Files (*.java);;JavaScript Files (*.js);;JSON Files (*.json);;"
            "Markdown Files (*.md);;PDF Files (*.pdf);;PHP Files (*.php);;"
            "PowerPoint Files (*.pptx);;Python Files (*.py);;Ruby Files (*.rb);;"
            "Shell Scripts (*.sh);;TeX Files (*.tex);;TypeScript Files (*.ts);;"
            "Text Files (*.txt);;All Files (*)"
        )
        if dialog.exec_():
            selected_files = dialog.selectedFiles()

            self.update_custommodals_SuccessModal2(f"Selected files!!")

            self.thread_Vectorstoreinassistant_AgentCreate = QProcessCreateVectorStoreByUser(selected_files, self.app1, self.client)
            self.thread_Vectorstoreinassistant_AgentCreate.vectorsignal.connect(self.update_Vectorstoreinassistant_AgentCreate)
                            
            self.thread_Vectorstoreinassistant_AgentCreate.ModalSucess.connect(self.update_custommodals_SuccessModal2)
            self.thread_Vectorstoreinassistant_AgentCreate.ModalInfo.connect(self.update_custommodals_info)

            self.thread_Vectorstoreinassistant_AgentCreate.start()


    def upload1fileinthread(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the files that will go to vector storage")
        dialog.setNameFilter(
            "C Files (*.c);;C++ Files (*.cpp);;C# Files (*.cs);;CSS Files (*.css);;"
            "Word Documents (*.doc *.docx);;Go Files (*.go);;HTML Files (*.html);;"
            "Java Files (*.java);;JavaScript Files (*.js);;JSON Files (*.json);;"
            "Markdown Files (*.md);;PDF Files (*.pdf);;PHP Files (*.php);;"
            "PowerPoint Files (*.pptx);;Python Files (*.py);;Ruby Files (*.rb);;"
            "Shell Scripts (*.sh);;TeX Files (*.tex);;TypeScript Files (*.ts);;"
            "Text Files (*.txt);;All Files (*)"
        )
        if dialog.exec_():
            selected_files = dialog.selectedFiles()[0]
            self.update_custommodals_SuccessModal2(f"Selected files!!")
            self.Upload1fileinThread_AgentCreate.clear()
            self.Upload1fileinThread_AgentCreate.setPlainText(f"'{selected_files}'")
            



    def upload1fileinmessage(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the files that will go to vector storage")
        dialog.setNameFilter(
            "C Files (*.c);;C++ Files (*.cpp);;C# Files (*.cs);;CSS Files (*.css);;"
            "Word Documents (*.doc *.docx);;Go Files (*.go);;HTML Files (*.html);;"
            "Java Files (*.java);;JavaScript Files (*.js);;JSON Files (*.json);;"
            "Markdown Files (*.md);;PDF Files (*.pdf);;PHP Files (*.php);;"
            "PowerPoint Files (*.pptx);;Python Files (*.py);;Ruby Files (*.rb);;"
            "Shell Scripts (*.sh);;TeX Files (*.tex);;TypeScript Files (*.ts);;"
            "Text Files (*.txt);;All Files (*)"
        )

        if dialog.exec_():
            selected_files = dialog.selectedFiles()[0]
            self.update_custommodals_SuccessModal2(f"Selected files!!")
            self.Upload1fileinmessage_AgentCreate.clear()
            self.Upload1fileinmessage_AgentCreate.setPlainText(f"'{selected_files}'")
     
    def upload1imageforvisioninThread(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFile) 
        dialog.setWindowTitle("Select the files that will go to vector storage")
        dialog.setNameFilter("Imagens (*.png *.jpg *.jpeg);;Todos os Arquivos (*)")  # Filtro
        if dialog.exec_():
            selected_files = dialog.selectedFiles()[0]
            self.update_custommodals_SuccessModal2(f"Selected files!!")
            self.Upload1imageforvisioninThread_AgentCreate.clear()
            self.Upload1imageforvisioninThread_AgentCreate.setPlainText(f"'{selected_files}'")

    def uploadlistfileforcodeinterpreterinthread(self):
        dialog = QFileDialog()
        dialog.setStyleSheet(self.dialogsetStyleSheet())
        dialog.setFileMode(QFileDialog.ExistingFiles) 
        dialog.setWindowTitle("Select the files that will go to vector storage")
        dialog.setNameFilter(
            "C Files (*.c);;C++ Files (*.cpp);;C# Files (*.cs);;CSS Files (*.css);;"
            "Word Documents (*.doc *.docx);;Go Files (*.go);;HTML Files (*.html);;"
            "Java Files (*.java);;JavaScript Files (*.js);;JSON Files (*.json);;"
            "Markdown Files (*.md);;PDF Files (*.pdf);;PHP Files (*.php);;"
            "PowerPoint Files (*.pptx);;Python Files (*.py);;Ruby Files (*.rb);;"
            "Shell Scripts (*.sh);;TeX Files (*.tex);;TypeScript Files (*.ts);;"
            "Text Files (*.txt);;All Files (*)"
        )
        if dialog.exec_():
            selected_files = dialog.selectedFiles()
            self.update_custommodals_SuccessModal2(f"Selected files!!")
            self.Uploadlistfileforcodeinterpreterinthread_AgentCreate.clear()
            self.Uploadlistfileforcodeinterpreterinthread_AgentCreate.setPlainText(f"[{selected_files}]")



    def list_category_Agent(self):
        paths_category = [
            'Software_Requirements_Analysis',
            'Software_Planning',
            'Software_Documentation',
            'Software_Development',
            'Pre_Project',
            'Company_Managers',
            'Company_CEO',
        ]
        for path in paths_category:
            self.AgentCategory_AgentCreate.addItem(path)

    def create_instruction(self):
        if self.Instructioninput_create.toPlainText().strip():
            NameForInstruction_create = self.NameForInstruction_create.text()
            agent_target =  self.InstructionSelector_create.currentText()
            category_target = self.InstructionCategory_create.currentText()
            paths_category = self.init_paths_category()
            for path in paths_category:
                if path == category_target:
                    PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/{category_target}'))
                    file_path = os.path.join(PATH_caminho, f"{NameForInstruction_create}.py")
                    with open(file_path, 'w', encoding='utf-8') as file:
                        content = self.Instructioninput_create.toPlainText()
                        file.write(F'''instruction{agent_target} = """{content}"""\nadxitional_instructions_{agent_target} = None''')
                        file.close()

                    self.Instructioninput_create.clear()
                    self.update_custommodals_SuccessModal("Your Instruction Was Created", "bottom-left")
        else:
            self.update_custommodals_erro("The Instruction Field Is Empty.")

    def list_agents(self):
        paths_agents = self.init_paths_agents()
        for path in paths_agents:
            agentss = os.listdir(path)
            for agent in agentss:
                if agent == "DocGitHubData":
                    pass
                elif agent == "docs_uploaded.log":
                    pass
                else:
                    iagents_refact = agent.replace(".py", "").replace("__pycache__", "").strip()
                    if iagents_refact:
                        self.InstructionSelector_create.addItem(iagents_refact)
                        self.AgentViewCode.addItem(iagents_refact)

    def list_category_instruction(self):
        paths_category = [
            'Software_Requirements_Analysis',
            'Software_Planning',
            'Software_Documentation',
            'Software_Development',
            'Pre_Project',
            'Company_Managers',
            'Company_CEO',
        ]
        for path in paths_category:
            self.InstructionCategory_create.addItem(path)



    def change_instruction(self):
        if self.Instructioninput_edit.toPlainText().strip():
            instructionarqname = self.InstructionSelector.currentText()
            paths_instruct = self.init_paths()
            for path in paths_instruct:
                instructions = os.listdir(path)
                for instruct in instructions:
                    instruct_refact = instruct.replace(".py", "").replace("__pycache__", "").strip()
                    if instruct_refact == instructionarqname:
                        file_path = os.path.join(path, f"{instruct_refact}.py")
                        with open(file_path, 'w', encoding='utf-8') as file:
                            content = self.Instructioninput_edit.toPlainText()
                            file.write(content)
                            file.close()
                        self.update_custommodals_SuccessModal(description="Your Instruction Has Been Updated", pos="bottom-left")
                        self.Instructioninput_edit.clear()
                        self.open_and_read(file_path)
        else:
            self.update_custommodals_erro("The Instruction Field Is Empty.")


    def on_InstructionSelector_changed(self, text):
        #self.InstructionSelector.clear()
        # paths_instruct = self.init_paths()
        # for path in paths_instruct:
        #     instructions = os.listdir(path)
        #     for instruct in instructions:
        #         instruct_refact = instruct.replace(".py", "").replace("__pycache__", "").strip()
        #         if instruct_refact:
        #             self.InstructionSelector.addItem(instruct_refact)
        
        self.CurrentInstruction.clear()
        paths_instruct = self.init_paths()
        for path in paths_instruct:
            instructions = os.listdir(path)
            for instruct in instructions:
                if instruct == "__pycache__":
                    pass
                else:
                    instruct_refact = instruct.replace(".py", "").strip()
                    if instruct_refact == text:
                        file_path = os.path.join(path, f"{instruct_refact}.py")
                        with open(file_path, 'r', encoding='utf-8') as file:
                            content = file.read()
                            
                            self.CurrentInstruction.setPlainText(content)
                        return True



    def list_instruction(self):
        self.InstructionSelector.clear()
        paths_instruct = self.init_paths()
        for path in paths_instruct:
            instructions = os.listdir(path)
            for instruct in instructions:
                instruct_refact = instruct.replace(".py", "").replace("__pycache__", "").strip()
                if instruct_refact:
                    self.InstructionSelector.addItem(instruct_refact)


    def update_Vectorstoreinassistant_AgentCreate(self, VectorstoreID):
        self.Vectorstoreinassistant_AgentCreate.setPlainText(VectorstoreID)

    def update_VectorstoreinThread_AgentCreate(self, VectorstoreID):
        self.VectorstoreinThread_AgentCreate.setPlainText(VectorstoreID)

    def addShadow(self, widget):
        effect = QGraphicsDropShadowEffect(widget)
        effect.setColor(QColor(30, 30, 30, 200))
        effect.setBlurRadius(20)
        effect.setXOffset(0)
        effect.setYOffset(0)
        widget.setGraphicsEffect(effect)

        
    def adjust_height(self, element, max_height = 150, increase=True):
        base_height = 50
        current_text_length = len(element.toPlainText())
        if current_text_length == 0:
            element.setMinimumHeight(base_height)
            element.setMinimumHeight(base_height)
            element.setMaximumHeight(base_height)
            return
        if increase:
            extra_lines = (current_text_length // 40) * 15  
            new_height = base_height + extra_lines
            if new_height > max_height:
                new_height = max_height
        else:
            new_height = max(base_height, max_height - (current_text_length // 40) * 15)

        element.setMinimumHeight(new_height)
        element.setMaximumHeight(new_height)

    def init_paths_category(self):
        paths_category = [
            'Software_Requirements_Analysis',
            'Software_Planning',
            'Software_Documentation',
            'Software_Development',
            'Pre_Project',
            'Company_Managers',
            'Company_CEO',
        ]
        return paths_category
    
    def init_paths_agents(self):
        paths_instruct = [
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Requirements_Analysis')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Planning')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Documentation')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Software_Development')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Pre_Project')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Company_Managers')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/Agents/Company_CEO')),
        ]
        return paths_instruct
    
    def init_paths(self):
        paths_instruct = [
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Software_Requirements_Analysis')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Software_Planning')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Software_Documentation')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Software_Development')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Pre_Project')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Company_Managers')),
            os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI/CoreApp/SoftwareAI/Instructions/Company_CEO')),
        ]
        return paths_instruct
    


    def open_and_read(self, file_path):
        with open(file_path, 'r', encoding='utf-8') as file:
            content = file.read()
            self.CurrentInstruction.setPlainText(content)



    def update_custommodals_erro(self, description, pos='center-center'):

        myModal = QCustomModals.ErrorModal(
            title="Information", 
            parent=self.myStackedWidget,
            position=pos,
            closeIcon=":/feather/icons/feather/window_close.png",
            modalIcon=":/material_design/icons/material_design/info.png",
            description=description,
            isClosable=False,
            duration=3000
        )
        myModal.show()


    def update_custommodals_warning(self, description, pos='center-center'):

        myModal = QCustomModals.WarningModal(
            title="Information", 
            parent=self.myStackedWidget,
            position=pos,
            closeIcon=":/feather/icons/feather/window_close.png",
            modalIcon=":/material_design/icons/material_design/info.png",
            description=description,
            isClosable=False,
            duration=3000
        )
        myModal.show()


    def update_custommodals_info(self, description, pos='top-right'):

        myModal = QCustomModals.InformationModal(
            title="Information", 
            parent=self.myStackedWidget,
            position=pos,
            closeIcon=":/feather/icons/feather/window_close.png",
            modalIcon=":/material_design/icons/material_design/info.png",
            description=description,
            isClosable=False,
            duration=3000
        )
        myModal.show()

    def update_custommodals_SuccessModal2(self, description, pos='top-right'):

        myModal = QCustomModals.SuccessModal(
            title="Information", 
            parent=self.myStackedWidget,
            position=pos,
            closeIcon=":/feather/icons/feather/window_close.png",
            modalIcon=":/material_design/icons/material_design/info.png",
            description=description,
            isClosable=False,
            duration=5000
        )
        myModal.show()


    def update_custommodals_SuccessModal(self, description, pos='center-center'):

        myModal = QCustomModals.SuccessModal(
            title="Information", 
            parent=self.myStackedWidget,
            position=pos,
            closeIcon=":/feather/icons/feather/window_close.png",
            modalIcon=":/material_design/icons/material_design/info.png",
            description=description,
            isClosable=False,
            duration=5000
        )
        myModal.show()


    def display_user_message(self, message):
        # Adiciona a mensagem ao QTextEdit
        if self.chat_area:  # Verifique se o QTextEdit foi inicializado corretamente
            self.chat_area.append(message)  # Adiciona a mensagem ao campo de texto
        else:
            print("Erro: chat_area não está inicializado.")



    def is_image_file(self, file_name):
        # Verifica a extensão do arquivo
        image_extensions = ('.png', '.jpg', '.jpeg', '.bmp', '.gif')
        return file_name.lower().endswith(image_extensions)

    def get_machine_info(self):
        system_info = platform.uname()
        machine = system_info.node
        return machine

    
    def closeEvent(self, event):
        try:
            self.thread_processe_create_agent.stop()
        except:
            pass
        try:
            self.thread_process_create_openaikeys.stop()
        except:
            pass
        try:
            self.thread_process_create_GitHubKeys.stop()
        except:
            pass
        try:
            self.thread_process_create_Firebasekeys.stop()
        except:
            pass




        event.accept()






########################################################################
## EXECUTE APP
########################################################################
if __name__ == "__main__":
    app = QApplication(sys.argv)

    
    window = MainWindow()
    sys.exit(app.exec_())
########################################################################
## END===>
######################################################################## 



from setuptools import setup, find_packages

setup(
    name="softwareai",  # Nome do pacote
    version="0.05.73",  # Versão inicial
    description="SoftwareAI is a framework with the aim of creating a software/application development company/organization governed by AI, its objective is not just to create the software with updates, documentation, schedules and spreadsheets, SoftwareAI is capable of running a software company completely with all the teams that make up a software company",
    long_description=open("READMEPIP.md", encoding="utf-8").read(),  # Codificação UTF-8
    long_description_content_type="text/markdown",
    author="ualers",
    author_email="freitasalexandre810@gmail.com",
    url="https://github.com/SoftwareAI-Company/SoftwareAI",
    license="Apache License 2.0",
    packages=find_packages(),  # Encontra todos os pacotes automaticamente
    include_package_data=True,  # Inclui arquivos não-Python listados no MANIFEST.in
    install_requires=[
        "firebase-admin",
        "asyncio",
        "pandas",
        "tiktoken",
        "PyGithub",
        "requests",
        "python-dotenv",
        "gitpython",
        "openai"
    ],    
    entry_points={
        'console_scripts': [
            'softwareai-cli = softwareai.softwareai_cli:app',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",  # Classificador da licença Apache 2.0
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',  # Versão mínima do Python
)




#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *





class QProcessCreateOpenAItokens(QThread):
    ModalSucess = Signal(str)
    ModalInfo = Signal(str)
    finish = Signal()
    def __init__(self,
                openaikey,
                openainamefortoken_AgentKeys
            ):
        super().__init__()
        self.openaikey = openaikey
        self.openainamefortoken_AgentKeys = openainamefortoken_AgentKeys

    def run(self):


        PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'../../../CoreApp/KeysOpenAI'))
        file_path = os.path.join(PATH_caminho, f"keys.py")
        with open(file_path, 'a', encoding='utf-8') as file:
            file.write(F'''

class OpenAIKeys{self.openainamefortoken_AgentKeys.replace(" ", "_")}:
    def keys():
        companyname = "{self.openainamefortoken_AgentKeys.replace(" ", "_")}"
        str_key = "{self.openaikey}"
        return str_key
    


            ''')
            file.close()


        self.ModalSucess.emit(f"Your Key has been created")
        self.finish.emit()




import sys
import os


import struct
import re
import platform
import json
import ast
from firebase_admin import credentials, initialize_app, storage, db, delete_app
from datetime import datetime


# Caminho absoluto para o diretório onde SoftwareAI está localizado (raiz do projeto)
caminho_raiz = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../SoftwareAI'))
sys.path.append(caminho_raiz)

from softwareai.CoreUi.Chat.Chat.Formatmessage import format_message



from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


from typing import Optional, List, Union
from typing_extensions import override



class QReadOpenAi(QThread):
    Threadlabel = Signal(str)
    Tokenslabel = Signal(int)
    htmlchat = Signal(str)
    code = Signal(str)
    messagesignal = Signal(str)
    finish = Signal()
    def __init__(self, agent, appx, client
                ):
        super().__init__()
        
        self.agent = agent
        self.appx = appx
        self.client = client

    def run(self):
        agent = self.agent
        appx = self.appx
        client = self.client
        refai_org_assistant_id = db.reference(f'ai_org_assistant_id/User_{agent}', app=appx)
        dataai_org_assistant_id = refai_org_assistant_id.get()
        assistant_id = dataai_org_assistant_id['assistant_id']
        assistant_idstr =  str(assistant_id)
        self.messagesignal.emit("Agent fetched\nfrom firebase")
        refai_org_thread_Id = db.reference(f'ai_org_thread_Id/User_{agent}', app=appx)
        datarefai_org_thread_Id = refai_org_thread_Id.get()
        thread_Id = datarefai_org_thread_Id['thread_id']
        self.Threadlabel.emit(thread_Id)
        self.messagesignal.emit("Thread fetched\nfrom firebase")
        run_status = client.beta.threads.runs.list(
            thread_id=thread_Id
        )
        jsonmodel = run_status.model_dump_json()
        with open("Cache/model_dump_json.json", 'w') as file:
            file.write(jsonmodel)
        with open("Cache/model_dump_json.json", "r") as file:
            data = json.load(file)  
        total_tokens_list = []
        prompt_tokens_list = []
        completion_tokens_list = []
        for run in data['data']:
            total_tokens = run['usage']['total_tokens']
            prompt_tokens = run['usage']['prompt_tokens']
            completion_tokens = run['usage']['completion_tokens']
            total_tokens_list.append(total_tokens)
            prompt_tokens_list.append(prompt_tokens)
            completion_tokens_list.append(completion_tokens)

        self.messagesignal.emit("Thread\nRemembered")

        total_tokens = sum(total_tokens_list)
        self.Tokenslabel.emit(total_tokens)
        prompt_tokens = sum(prompt_tokens_list)
        completion_tokens = sum(completion_tokens_list)
        contador_1 = 0
        ids = [item['id'] for item in data['data']]
        for id in ids:
            run_status = client.beta.threads.runs.retrieve(
                thread_id=thread_Id,
                run_id=id
            )

            messages = client.beta.threads.messages.list(
                thread_id=thread_Id
            )
            for message in messages:
                for mensagem_contexto in message.content:
                    valor_texto = mensagem_contexto.text.value
                    print(valor_texto)
                    self.emit_message("messagethread", valor_texto)
                    contador_1 += 1
                    if contador_1 == 5:
                        break
                break
            break
        self.finish.emit()

    def extract_python_code(self, text: str) -> Optional[str]:
        pattern = r"```python\s*(.*?)\s*```"
        match = re.search(pattern, text, re.DOTALL)
        
        if match:
            code = match.group(1).strip()
            self.code.emit(code)
            return code
        return None
    
    def emit_message(self, sender, message):
        self.extract_python_code(message)
        formatted_message = format_message(message)

        if sender == "Chat":
            formatted_message = (
                f'<div style="display: flex; justify-content: flex-end;">'
                f'<div style="background-color: #d4f4dd; color: black; padding: 8px; border-radius: 8px; '
                f'margin: 5px; max-width: 70%;"><b>user:</b> {formatted_message}</div>'
                f'</div>'
            )
        elif sender == "messagethread":
            formatted_message = (
                f'<div style="display: flex; justify-content: flex-start;">'
                f'<div style=color: black; padding: 8px; border-radius: 8px; '
                f'margin: 5px; max-width: 70%;"><b>Message Remembered From Thread:</b> <br>{formatted_message}<br></div>'
                f'</div>'
            )

        self.htmlchat.emit(formatted_message)

    def stop(self):
        self.running = False
        






#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *




class QProcessCreateGitHubKeys(QThread):
    ModalSucess = Signal(str)
    ModalInfo = Signal(str)
    finish = Signal()
    def __init__(self,
                githuusername_AgentKeys,
                githubtoken_AgentKeys
            ):
        super().__init__()
        self.githuusername_AgentKeys = githuusername_AgentKeys
        self.githubtoken_AgentKeys = githubtoken_AgentKeys

    def run(self):


        PATH_caminho = os.path.abspath(os.path.join(os.path.dirname(__file__), f'../../../CoreApp/KeysGitHub'))
        file_path = os.path.join(PATH_caminho, f"keys.py")
        with open(file_path, 'a', encoding='utf-8') as file:
            file.write(F'''


class GithubKeys{self.githuusername_AgentKeys.replace(" ", "_")}:
                       
    def {self.githuusername_AgentKeys.replace(" ", "_")}_github_keys():
        github_username = "{self.githuusername_AgentKeys}"
        github_token = "{self.githubtoken_AgentKeys}"
        return github_username, github_token



            ''')
            file.close()


        self.ModalSucess.emit(f"Your Key has been created")
        self.finish.emit()





#########################################
# IMPORT SoftwareAI Core
from softwareai.CoreApp._init_core_ import * 
#########################################
# IMPORT SoftwareAI Libs 
from softwareai.CoreApp._init_libs_ import *
#########################################
# IMPORT SoftwareAI All Paths 
from softwareai.CoreApp._init_paths_ import *
#########################################
# IMPORT SoftwareAI Instructions
from softwareai.CoreApp._init_Instructions_ import *
#########################################
# IMPORT SoftwareAI Tools
from softwareai.CoreApp._init_tools_ import *
#########################################
# IMPORT SoftwareAI keys
from softwareai.CoreApp._init_keys_ import *
#########################################
# IMPORT SoftwareAI _init_environment_
from softwareai.CoreApp._init_environment_ import init_env



class GearAssist:
    """GearAssist é um agente tecnico de suporte construido com softwareai, uma ferramenta avançada de rastreamento e resolução de problemas técnicos de software .\nhttps://github.com/SoftwareAI-Company/SoftwareAI/blob/main/Docs/Agents/GearAssist.md"""
    def __init__(self
                ):
        pass

    def GearAssist_Technical_Support(self, 
                                    Ticketid,
                                    appcompany,
                                    app_product,
                                    client,
                                    AutenticateAgent,
                                    ResponseAgent,
                                ):

        vectorstore_in_assistant = None,
        vectorstore_in_Thread = None,
        Upload_1_file_in_thread = None,
        Upload_1_file_in_message = None,
        Upload_1_image_for_vision_in_thread = None,
        Upload_list_for_code_interpreter_in_thread = None,
        path_to_save_report=os.path.join(os.path.dirname(__file__), 'technical_report', 'nomegeradoautomaticamente.md') 

        tools_GearAssist = [
            {"type": "file_search"},
            {
                "type": "function",
                "function": {
                    "name": "AutoGetLoggerUser",
                    "description": "Realiza o rastreamento de problemas tecnicos no software do usuario usando so Ticket id, Retornando um dicionario json com o processo do usuario em um range de 1 dia antes da abertura do ticket ",
                    "parameters": {
                        "type": "object",
                        "properties": {
                            "ticketid": {
                                "type": "string",
                                "description": "Ticket id"
                            }
                        },
                        "required": ["ticketid"]
                    }
                }
            },
            {
                "type": "function",
                "function": {
                    "name": "autosave",
                    "description": "Salva um codigo python em um caminho",
                    "parameters": {
                        "type": "object",
                        "properties": {
                            "code": {
                                "type": "string",
                                "description": "codigo"
                            },
                            "path": {
                                "type": "string",
                                "description": "Caminho do codigo"
                            }
                        },
                        "required": ["code","path"]
                    }
                }
            }

            
        ]

        instruction_GearAssist = f"""
        Meu nome é **GearAssist**, sou uma ferramenta de rastreamento e resolução de problemas técnicos avançada. Minha responsabilidade é identificar, rastrear e fornecer insights sobre problemas reportados pelos usuários, utilizando o Ticket ID associado.

        ### **Minhas Responsabilidades:**  

        1. **Receber e Validar o Ticket ID:**  
        - Recebo um **Ticket ID único** fornecido pelo usuário ou sistema.  
        - **Valido a integridade** do Ticket ID para garantir que ele seja utilizável.  

        2. **Acessar Informações Associadas ao Ticket ID:**  
        - Utilizo a função **AutoGetLoggerUser** para acessar os dados relacionados ao Ticket ID.  
        - A função retorna:  
            - **Descrição do Problema:** Uma explicação fornecida pelo cliente ou sistema sobre o que está acontecendo.  
            - **Dicionário de Logs:** Logs detalhados do sistema para análise técnica.  

        3. **Analisar a Descrição do Problema:**  
        - Inicio o diagnóstico a partir da descrição fornecida, destacando os pontos-chave mencionados pelo usuário.  

        4. **Analisar Logs e Identificar Problemas:**  
        - Examino cuidadosamente o dicionário de logs retornado para identificar:  
            - **Erros técnicos** (exceções, falhas ou mensagens de erro).  
            - **Comportamentos anômalos**.  
            - **Padrões recorrentes** de problemas que podem indicar falhas maiores.  

        5. **Fornecer Diagnóstico Detalhado:**  
        - Gero um relatório completo e estruturado com base na descrição e nos logs analisados, contendo:  
            - **Resumo do Problema** (baseado na descrição fornecida).  
            - **Diagnóstico Técnico** (análise dos logs).  
            - **Impacto Potencial** no sistema ou experiência do usuário.  
            - **Recomendações Iniciais** para resolução.  

        6. **Salvar o Relatório em Formato `.md`:**  
        - Utilizo a função **autosave** para salvar o relatório gerado como um arquivo `.md` no caminho especificado.  
        - **Detalhes da Função:**  
            - **code:** O conteúdo do relatório detalhado gerado.  
            - **path:** Caminho onde o arquivo será salvo, com o nome gerado automaticamente:  
            `"{path_to_save_report}"`

        7. **Manter Clareza e Organização:**  
        - O relatório é estruturado, claro e conciso, garantindo que todas as partes interessadas possam entender.  
        - **Evito redundância** e me concentro nos aspectos mais relevantes.

        ---

        ### **Funções Disponíveis:**  

        #### **AutoGetLoggerUser**  
        - **Descrição:** Retorna informações detalhadas sobre um problema técnico com base no Ticket ID.  
        - **Entrada:**  
        - **Ticket ID:** Identificador único fornecido para rastrear o problema.  
        - **Saída:**  
        - **Descrição do Problema:** Uma explicação textual do problema relatado pelo cliente.  
        - **Dicionário de Logs:** Logs detalhados do sistema.  

        #### **autosave**  
        - **Descrição:** Salva o conteúdo gerado em um arquivo no formato especificado.  
        - **Entrada:**  
        - **code:** O relatório detalhado e estruturado gerado.  
        - **path:** O caminho para salvar o arquivo, incluindo o nome e extensão.  

        ---

        ### **Fluxo de Trabalho Automatizado:**  

        1. **Receber o Ticket ID:**  
        - Aceito o **Ticket ID** como entrada inicial.  

        2. **Obter Informações Associadas:**  
        - Utilizo a função **AutoGetLoggerUser** para recuperar:  
            - **Descrição do Problema**.  
            - **Dicionário de Logs**.  

        3. **Analisar e Diagnosticar:**  
        - **Descrição do Problema:**  
            - Identifico palavras-chave e pontos importantes mencionados pelo cliente.  
        - **Dicionário de Logs:**  
            - Examino os logs detalhadamente para identificar erros e possíveis causas do problema.  
        - **Relatório Completo:**  
            - Gero um relatório estruturado com:  
            - **Resumo do Problema** (com base na descrição fornecida).  
            - **Diagnóstico Técnico** (com base nos logs).  
            - **Impacto Potencial**.  
            - **Recomendações Iniciais para Resolução**.  

        4. **Salvar o Relatório:**  
        - Salvo o relatório gerado utilizando a função **autosave**, especificando o conteúdo e o caminho do arquivo.  

        5. **Retornar Confirmação:**  
        - Após salvar o relatório, confirmo que o arquivo foi gerado e salvo com sucesso.  

        ---

        Regras Rígidas:
            A integridade do diagnóstico é crucial: Certifique-se de que todos os problemas relevantes foram abordados.
            Salve sempre o relatório completo em formato .md utilizando a função autosave.
            Mantenha clareza e objetividade no relatório de diagnóstico.
            Evite omissões: O diagnóstico deve ser completo e fornecer um entendimento claro do problema.
        """
        
        key = "GearAssist_Technical_Support"
        nameassistant = "GearAssist Technical Support"
        model_select = "gpt-4o-mini-2024-07-18"

        GearAssist_Technical_Support_AI, instructionsassistant, nameassistant, model_select = AutenticateAgent.create_or_auth_AI(appcompany, client, key, instruction_GearAssist, nameassistant, model_select, tools_GearAssist)

        mensaxgem = f"""
        Ticketid:{Ticketid}
        """  
        adxitional_instructions = ""
        mensaxgemfinal = mensaxgem
        
        response, total_tokens, prompt_tokens, completion_tokens = ResponseAgent.ResponseAgent_message_with_assistants(
                                                                mensagem=mensaxgemfinal,
                                                                agent_id=GearAssist_Technical_Support_AI, 
                                                                key=key,
                                                                app1=appcompany,
                                                                app_product=app_product,
                                                                client=client,
                                                                tools=tools_GearAssist,
                                                                model_select=model_select,
                                                                aditional_instructions=adxitional_instructions
                                                                )
                
        return response



